@@ -28,13 +28,6 @@
 #include "mem.h"
 #include "filetype.h"
 
-
-int cfg_acl_add(char *directive, char **argv, void *setdata);
-struct ci_conf_entry acl_conf_variables[] = {
-     {"acl", NULL, cfg_acl_add, NULL},
-     {NULL, NULL, NULL, NULL}
-};
-
 /*standard acl types */
 
 /*Spec types:
@@ -243,7 +236,7 @@ const ci_acl_spec_t *ci_access_entry_add_acl(ci_access_entry_t *access_entry, co
      return acl;
 }
 
-int ci_access_entry_add_acl_by_name(ci_access_entry_t *access_entry, char *acl_name){
+int ci_access_entry_add_acl_by_name(ci_access_entry_t *access_entry, const char *acl_name){
      const ci_acl_spec_t *acl;
      int negate = 0;
      if (acl_name[0] == '!') {
@@ -265,7 +258,7 @@ int ci_access_entry_add_acl_by_name(ci_access_entry_t *access_entry, char *acl_n
 /*********************************************************************************/
 /*ci_acl_spec functions                                                          */
 
-ci_acl_spec_t *  ci_acl_spec_new(char *name, char *type, char *param, struct ci_acl_type_list *list, ci_acl_spec_t **spec_list)
+ci_acl_spec_t *  ci_acl_spec_new(const char *name, const char *type, const char *param, struct ci_acl_type_list *list, ci_acl_spec_t **spec_list)
 {
      ci_acl_spec_t *spec,*cur;
      const ci_acl_type_t *acl_type;
@@ -279,8 +272,10 @@ ci_acl_spec_t *  ci_acl_spec_new(char *name, char *type, char *param, struct ci_
      strncpy(spec->name, name, MAX_NAME_LEN);
      spec->name[MAX_NAME_LEN] = '\0';
      if(param) {
-	 if (!(spec->parameter = strdup(param)))
-	     return NULL;/*leak but if a simple strdup fails who cares....*/
+	 if (!(spec->parameter = strdup(param))) {
+             free(spec);
+	     return NULL;
+         }
      }
      else
 	 spec->parameter = NULL;
@@ -301,7 +296,7 @@ ci_acl_spec_t *  ci_acl_spec_new(char *name, char *type, char *param, struct ci_
      return spec;
 }
 
-ci_acl_data_t *ci_acl_spec_new_data(ci_acl_spec_t *spec, char *val)
+ci_acl_data_t *ci_acl_spec_new_data(ci_acl_spec_t *spec, const char *val)
 {
      ci_acl_data_t *new_data, *list;
      const ci_type_ops_t *ops;
@@ -316,6 +311,10 @@ ci_acl_data_t *ci_acl_spec_new_data(ci_acl_spec_t *spec, char *val)
 	 return NULL;
 
      new_data = malloc(sizeof(ci_acl_data_t));
+     if (!new_data) {
+         ops->free(data, default_allocator);
+         return NULL;
+     }
      new_data->data = data;
      new_data->next = NULL;
      if ((list=spec->data) != NULL) {
@@ -386,6 +385,7 @@ int ci_acl_typelist_init(struct ci_acl_type_list *list)
 int ci_acl_typelist_add(struct ci_acl_type_list *list, const ci_acl_type_t *type)
 {
      ci_acl_type_t *cur;
+     ci_acl_type_t *nl = NULL;
 
      if (!type->name)
        return 0;
@@ -397,8 +397,13 @@ int ci_acl_typelist_add(struct ci_acl_type_list *list, const ci_acl_type_t *type
 
      if (list->acl_type_list_num == list->acl_type_list_size) {
 	  list->acl_type_list_size += STEP;
-	  list->acl_type_list = realloc((void *)list->acl_type_list, 
+          nl = realloc((void *)list->acl_type_list, 
 					list->acl_type_list_size*sizeof(ci_acl_type_t));
+          if (!nl) {
+              ci_debug_printf(1, "Failed to allocate more space for new ci_acl_typr_t\n");
+              return 0;
+          }
+          list->acl_type_list = nl;
      }
 
      cur = &(list->acl_type_list[list->acl_type_list_num]);
@@ -459,7 +464,7 @@ int request_match_specslist(ci_request_t *req, const struct ci_specs_list *spec_
 {
     const ci_acl_spec_t *spec;
     const ci_acl_type_t *type;
-    int ret, negate;
+    int ret, negate, check_result;
     void *test_data;
 
     ret = 1;
@@ -473,9 +478,10 @@ int request_match_specslist(ci_request_t *req, const struct ci_specs_list *spec_
 	    return 0;
 	}
 
-	if (spec_data_check(spec, test_data)==0 && negate==0)
+        check_result = spec_data_check(spec, test_data);
+	if (check_result==0 && negate==0)
 	    ret = 0;
-	else if (spec_data_check(spec, test_data)!=0 && negate!=0)
+	else if (check_result!=0 && negate!=0)
 	    ret = 0;
 
 	if (type->free_test_data)
@@ -559,58 +565,58 @@ int ci_acl_type_add(const ci_acl_type_t *type)
     return ci_acl_typelist_add(&types_list, type);
 }
 
-int cfg_acl_add(char *directive, char **argv, void *setdata)
+int ci_acl_add_data(const char *name, const char *type, const char *data)
 {
-    char *s, *acl_name,*acl_type, *param=NULL;
-     int argc;
-     ci_acl_spec_t *spec;
-     const ci_acl_type_t *spec_type;
+    ci_acl_spec_t *spec;
+    const ci_acl_type_t *spec_type;
+    char *s, *param = NULL, *acl_type;
 
-     if (!argv[0] || !argv[1] || !argv[2]) /* at least an argument */
-	 return 0;
-     
-     
-     acl_name = argv[0];
-     acl_type = argv[1];
-     if ((s=strchr(argv[1],'{')) != NULL) {
-	 *s='\0';
-	 param=s+1;
-	 if((s=strchr(param,'}')) != NULL)
-	     *s= '\0';
-     }
+    acl_type = strdup(type);
+    if (!acl_type) {
+        ci_debug_printf(1, "cfg_acl_add: error strduping!\n"); 
+        return 0;
+    }
 
-     if ((spec=ci_acl_spec_search(specs_list, acl_name)) != NULL){
-	  spec_type = ci_acl_typelist_search(&types_list, acl_type);
-	  if(spec_type != spec->type){
-	       ci_debug_printf(1, "The acl type:%s does not much with type of existing acl \"%s\"", 
-			       acl_type,acl_name);
-	       return 0;
-	  }
-     }
-     else {
-	 ci_debug_printf(2, "New ACL with name:%s and  ACL Type: %s{%s}\n", argv[0], argv[1],param?param:"NULL");
-	  spec = ci_acl_spec_new(acl_name, acl_type, param, &types_list, &specs_list);
-     }
+    s = acl_type;
+    if ((s=strchr(s,'{')) != NULL) {
+        *s='\0';
+        param=s+1;
+        if((s=strchr(param,'}')) != NULL)
+            *s= '\0';
+    }
 
-     if (!spec){
-	  ci_debug_printf(1, "Error in acl:%s! Maybe the acl type \"%s\" does not exists!\n",
-			  acl_name,acl_type);
-	  return 0;
-     }
-     for (argc = 2; argv[argc] != NULL; argc++){
-	  ci_debug_printf(2, "Adding to acl %s the data %s\n", acl_name, argv[argc]);
-	  ci_acl_spec_new_data(spec, argv[argc]);
-     }	  
-     return 1;
+    if ((spec = ci_acl_spec_search(specs_list, name)) != NULL){
+        spec_type = ci_acl_type_search(acl_type);
+        if(spec_type != spec->type){
+            ci_debug_printf(1, "The acl type:%s does not match with type of existing acl \"%s\"", 
+                            acl_type, name);
+            free(acl_type);
+            return 0;
+        }
+    }
+    else {
+        spec = ci_acl_spec_new(name, acl_type, param, &types_list, &specs_list);
+    }
+    free(acl_type);
+
+    if (!spec){
+        ci_debug_printf(1, "Error in acl:%s! Maybe the acl type \"%s\" does not exists!\n",
+                        name, acl_type);
+        return 0;
+    }
+    ci_acl_spec_new_data(spec, data);
+
+    return 1;
 }
 
 /******************************************************/
 /* Some acl_type methods implementation               */
 #if HAVE_REGEX
 
-char *get_header(ci_headers_list_t *headers, char *head)
+const char *get_header(ci_headers_list_t *headers, char *head)
 {
-    char *val, *buf;
+    const char *val;
+    char *buf;
     int i;
 
     if(!headers || !head)
@@ -622,20 +628,20 @@ char *get_header(ci_headers_list_t *headers, char *head)
     if (!headers->packed) /*The headers are not packed, so it is NULL terminated*/
 	return val;
 
-    /*assume that an 1024 buffer is enough for a header value*/
-    if (!(buf = ci_buffer_alloc(1024)))
+    /*assume that an 8k buffer is enough for a header value*/
+    if (!(buf = ci_buffer_alloc(8192)))
 	return NULL;
 
-     for(i=0;i<1023 && *val!= '\0' && *val != '\r' && *val!='\n'; i++,val++) 
+     for(i=0;i<8191 && *val!= '\0' && *val != '\r' && *val!='\n'; i++,val++) 
         buf[i] = *val;
-     buf[1023]='\0';
+     buf[8191]='\0';
 
      return buf;
 }
 
 void release_header_value(ci_headers_list_t *headers, char *head)
 {
-    if (headers->packed) /*The headers are packed, we have allocated buffer*/
+    if (headers && headers->packed && head) /*The headers are packed, we have allocated buffer*/
 	ci_buffer_free(head);
 }
 
@@ -702,7 +708,11 @@ void *get_data_type(ci_request_t *req, char *param){
     type = ci_magic_req_data_type(req, &isenc);
     if (type < 0)
         return NULL;
+
     ret_type = malloc(sizeof(unsigned int));
+    if (!ret_type)
+        return NULL;
+
     *ret_type = type;
     return (void *)ret_type;
 }
