@@ -22,6 +22,7 @@
 #include "debug.h"
 #include "ci_threads.h"
 #include "lookup_table.h"
+#include "array.h"
 #include "cache.h"
 #include <assert.h>
 
@@ -86,8 +87,14 @@ struct ci_cache *ci_cache_build( unsigned int cache_size,
     /*until we are going to create an allocator which can allocate/release from 
      continues memory blocks like those we have in shared memory*/
     allocator = ci_create_os_allocator();
-    
+    if (!allocator)
+        return NULL;
+
     cache = malloc(sizeof(struct ci_cache));
+    if (!cache) {
+        ci_mem_allocator_destroy(allocator);
+        return NULL;
+    }
     cache->key_ops = key_ops;
     cache->allocator = allocator;
 //    cache->data_release = data_release;
@@ -99,6 +106,11 @@ struct ci_cache *ci_cache_build( unsigned int cache_size,
     cache->copy_from = copy_from_cache;
     
     cache->first_queue_entry = (struct ci_cache_entry *)allocator->alloc(allocator, sizeof(struct ci_cache_entry)); 
+    if (!cache->first_queue_entry) {
+        ci_mem_allocator_destroy(allocator);
+        free(cache);
+        return NULL;
+    }
     cache->last_queue_entry = cache->first_queue_entry;
     cache->last_queue_entry->hnext=NULL;
     cache->last_queue_entry->qnext = NULL;
@@ -108,6 +120,12 @@ struct ci_cache *ci_cache_build( unsigned int cache_size,
     cache->last_queue_entry->hash = 0;
     for (i=0; i < cache_items-1; i++) {
 	cache->last_queue_entry->qnext=(struct ci_cache_entry *)allocator->alloc(allocator, sizeof(struct ci_cache_entry));
+        if (!cache->last_queue_entry->qnext) {
+            /*we are leaking here the cache->first_queue_entry elements. TODO...*/
+            ci_mem_allocator_destroy(allocator);
+            free(cache);
+            return NULL;            
+        }
 	cache->last_queue_entry=cache->last_queue_entry->qnext;
 	cache->last_queue_entry->hnext=NULL;
         cache->last_queue_entry->qnext = NULL;
@@ -126,6 +144,12 @@ struct ci_cache *ci_cache_build( unsigned int cache_size,
     }
     ci_debug_printf(7,"Hash size: %d\n",new_hash_size);
     cache->hash_table=(struct ci_cache_entry **)allocator->alloc(allocator, (new_hash_size+1)*sizeof(struct ci_cache_entry *));
+    if (!cache->hash_table) {
+        /*we are leaking here the cache->first_queue_entry elements. TODO...*/
+        ci_mem_allocator_destroy(allocator);
+        free(cache);
+        return NULL;
+    }
     memset(cache->hash_table,0,(new_hash_size+1)*sizeof(struct ci_cache_entry *));
     cache->hash_table_size = new_hash_size; 
     cache->ttl = ttl;
@@ -142,7 +166,7 @@ void ci_cache_destroy(struct ci_cache *cache)
 	cache->first_queue_entry = cache->first_queue_entry->qnext;
 	if (e->key)
 	    cache->key_ops->free(e->key, cache->allocator);
-	if (e->val)
+	if (e->val && e->val_size > 0)
 	    cache->allocator->free(cache->allocator, e->val);
 	cache->allocator->free(cache->allocator, e);
 	e = cache->first_queue_entry;
@@ -169,6 +193,7 @@ void *ci_cache_search(struct ci_cache *cache,void *key, void **val, ci_mem_alloc
 	    common_mutex_unlock(&cache->mtx);
 	    return key;
 	}
+        assert(e != e->hnext);
 	e = e->hnext;
     }
     common_mutex_unlock(&cache->mtx);
@@ -205,13 +230,13 @@ int ci_cache_update(struct ci_cache *cache, void *key, void *val) {
 	cache->key_ops->free(e->key, cache->allocator);
 	e->key = NULL;
     }
-    if (e->val) {
+    if (e->val && e->val_size > 0) {
 	cache->allocator->free(cache->allocator, e->val);
 	e->val = NULL;
     }
 
     /*If it is in the hash table remove it...*/
-    if(e->hash) {
+    {
         assert(e->hash <= cache->hash_table_size);
 	tmp = cache->hash_table[e->hash];
 	if(tmp == e)
@@ -221,11 +246,11 @@ int ci_cache_update(struct ci_cache *cache, void *key, void *val) {
 	    if(tmp->hnext)
 		tmp->hnext = tmp->hnext->hnext;
 	}
-	e->hash = 0;
     }
     
     e->hnext = NULL;
     e->time = 0;
+    e->hash = 0;
 
     /*I should implement a ci_type_ops::clone method. Maybe the memcpy is not enough....*/
     key_size = cache->key_ops->size(key);
@@ -269,3 +294,74 @@ int ci_cache_update(struct ci_cache *cache, void *key, void *val) {
 }
 
 
+void *ci_cache_store_vector_val(void *val, int *val_size, ci_mem_allocator_t *allocator)
+{
+    int  vector_data_size, vector_indx_size, i;
+    void *vector_data_start;
+    void *vector_data_end;
+    void *data, **data_indx;
+    ci_vector_t *v = (ci_vector_t *)val;
+
+    if (!val) {
+        *val_size = 0;
+        return NULL;
+    }
+
+    /*The vector data stored in a continue memory block which filled from 
+      bottom to up. So the last elements stored at the beggining of the 
+      memory block. */
+    vector_data_start = (void *)(v->items[v->count -1]);
+    vector_data_end = v->mem +v->max_size;
+    /*Assert that the vector stored in one memory block (eg it is not a ci_ptr_vector_t object)*/
+    assert(vector_data_start < vector_data_end && vector_data_start > (void *)v->mem);
+
+    /*compute the required memory for storing the vector*/
+    vector_data_size = vector_data_end - vector_data_start;
+    vector_indx_size = (v->count+1) * sizeof(void *);
+    *val_size = sizeof(size_t) + vector_indx_size + vector_data_size ;
+    
+    data = allocator->alloc(allocator, *val_size);
+    if (!data) {
+        ci_debug_printf(1, "store_str_vector_val: error allocation memory of size %d\n", *val_size);
+        return NULL;
+    }
+
+    /*store the size of vector*/
+    memcpy(data, &(v->max_size), sizeof(size_t));
+    data_indx = data + sizeof(size_t);
+    memcpy((void *)data_indx+vector_indx_size, vector_data_start, vector_data_size);
+
+    /*Store the relative position of the vector item to the index part*/
+    for(i = 0; v->items[i]!= NULL; i++)
+        data_indx[i] = (void *)(v->items[i] - vector_data_start + vector_indx_size);
+    data_indx[i] = NULL;
+
+    return data;
+}
+
+void *ci_cache_read_vector_val(void *val, int val_size, ci_mem_allocator_t *allocator)
+{
+    size_t vector_size, item_size;
+    int i;
+    ci_vector_t *v;
+    void **data_indx;
+
+    if (!val)
+        return NULL;
+
+    data_indx = (void **)(val + sizeof(size_t));
+    vector_size = *((size_t *)val);
+    v= ci_vector_create(vector_size);
+    
+    /*The items stores from bottom to top.
+      Compute the size of first item, which stored at the end of *val*/
+    item_size = val_size - sizeof(size_t) - (size_t)data_indx[0];
+    for(i=0; data_indx[i] != NULL; i++) {
+        ci_vector_add(v, (void *)((void *)data_indx+(size_t)data_indx[i]), item_size);
+        /*compute the item size of the next item*/
+        item_size = data_indx[i] - data_indx[i+1];
+    }
+
+    return v;
+}
+
