@@ -50,49 +50,71 @@ void *ci_cfg_alloc_mem(int size)
 /****************************************************************/
 /* Command line options implementation, function and structures */
 
-void ci_args_usage(char *progname, struct ci_options_entry *options)
+void ci_args_usage(const char *progname, struct ci_options_entry *options)
 {
      int i;
      printf("Usage : \n");
      printf("%s", progname);
-     for (i = 0; options[i].name != NULL; i++)
-          printf(" [%s %s]", options[i].name,
-                 (options[i].parameter == NULL ? "" : options[i].parameter));
+     for (i = 0; options[i].name != NULL; i++) {
+         if (options[i].name[0] == '$')
+             printf(" [file1] [file2] ...");
+         else
+             printf(" [%s %s]", options[i].name,
+                    (options[i].parameter == NULL ? "" : options[i].parameter));
+     }
      printf("\n\n");
      for (i = 0; options[i].name != NULL; i++)
-          printf("%s %s\t\t: %s\n", options[i].name,
-                 (options[i].parameter == NULL ? "\t" : options[i].parameter),
-                 options[i].msg);
-
+         if (options[i].name[0] == '$')
+             printf(" [file1] [file2] ...\t: %s\n", options[i].msg);
+         else
+             printf("%s %s\t\t: %s\n", options[i].name,
+                    (options[i].parameter == NULL ? "\t" : options[i].parameter),
+                    options[i].msg);
 }
 
 
-struct ci_options_entry *search_options_table(char *directive,
+struct ci_options_entry *search_options_table(const char *directive,
                                            struct ci_options_entry *options)
 {
      int i;
+     const char *option_search;
+     if (directive[0] != '-') 
+         option_search = "$$";
+     else
+         option_search = directive;
+
      for (i = 0; options[i].name != NULL; i++) {
-          if (0 == strcmp(directive, options[i].name))
+          if (0 == strcmp(option_search, options[i].name))
                return &options[i];
      }
      return NULL;
 }
 
 
-int ci_args_apply(int argc, char **argv, struct ci_options_entry *options)
+int ci_args_apply(int argc, char *argv[], struct ci_options_entry *options)
 {
      int i;
      struct ci_options_entry *entry;
+     const char *act_args[2];
+     act_args[1] = NULL;
      for (i = 1; i < argc; i++) {
           if ((entry = search_options_table(argv[i], options)) == NULL)
                return 0;
           if (entry->parameter) {
                if (++i >= argc)
                     return 0;
-               (*(entry->action)) (entry->name, argv + i, entry->data);
+               act_args[0] = argv[i];
+               (*(entry->action)) (entry->name, act_args, entry->data);
+          }
+          else {
+              /*maybe is the "$$" directive ....*/
+              if (strcmp(entry->name, "$$") == 0) {
+                  act_args[0] = argv[i];
+                  (*(entry->action)) (entry->name, act_args, entry->data);
+              }
+              else
+                  (*(entry->action)) (entry->name, NULL, entry->data);
           }
-          else
-               (*(entry->action)) (entry->name, NULL, entry->data);
      }
      return 1;
 }
@@ -101,7 +123,7 @@ int ci_args_apply(int argc, char **argv, struct ci_options_entry *options)
 /****************************************************************************/
 /*Various functions for setting parameters from command line or config file */
 
-int ci_cfg_set_int(char *directive, char **argv, void *setdata)
+int ci_cfg_set_int(const char *directive, const char **argv, void *setdata)
 {
      int val = 0;
      char *end;
@@ -125,7 +147,7 @@ int ci_cfg_set_int(char *directive, char **argv, void *setdata)
      return 1;
 }
 
-int ci_cfg_set_str(char *directive, char **argv, void *setdata)
+int ci_cfg_set_str(const char *directive, const char **argv, void *setdata)
 {
      if (setdata == NULL)
           return 0;
@@ -144,7 +166,7 @@ int ci_cfg_set_str(char *directive, char **argv, void *setdata)
      return 1;
 }
 
-int ci_cfg_onoff(char *directive, char **argv, void *setdata)
+int ci_cfg_onoff(const char *directive, const char **argv, void *setdata)
 {
      if (setdata == NULL)
           return 0;
@@ -167,7 +189,7 @@ int ci_cfg_onoff(char *directive, char **argv, void *setdata)
 }
 
 
-int ci_cfg_disable(char *directive, char **argv, void *setdata)
+int ci_cfg_disable(const char *directive, const char **argv, void *setdata)
 {
      if (setdata == NULL)
           return 0;
@@ -178,7 +200,7 @@ int ci_cfg_disable(char *directive, char **argv, void *setdata)
 
 }
 
-int ci_cfg_enable(char *directive, char **argv, void *setdata)
+int ci_cfg_enable(const char *directive, const char **argv, void *setdata)
 {
      if (setdata == NULL)
           return 0;
@@ -188,7 +210,7 @@ int ci_cfg_enable(char *directive, char **argv, void *setdata)
      return 1;
 }
 
-int ci_cfg_size_off(char *directive, char **argv, void *setdata)
+int ci_cfg_size_off(const char *directive, const char **argv, void *setdata)
 {
      ci_off_t val = 0;
      char *end;
@@ -220,7 +242,7 @@ int ci_cfg_size_off(char *directive, char **argv, void *setdata)
 }
 
 
-int ci_cfg_size_long(char *directive, char **argv, void *setdata)
+int ci_cfg_size_long(const char *directive, const char **argv, void *setdata)
 {
      long int val = 0;
      char *end;
