@@ -30,7 +30,10 @@
 #include "cfg_param.h"
 #include "commands.h"
 #include "acl.h"
+#include "txtTemplate.h"
+#include "registry.h"
 
+#define MAX_INCLUDE_LEVEL 5
 #define LINESIZE 8192
 #define MAX_DIRECTIVE_SIZE 80
 #define MAX_ARGS 50
@@ -57,7 +60,12 @@ struct ci_server_conf CONF = {
      SERVDIR,                   /*SERVICES_DIR */
      MODSDIR,                   /*MODULES_DIR */
      NULL,                      /*SERVER_ADMIN*/
-     NULL                       /*SERVER_NAME*/
+     NULL,                      /*SERVER_NAME*/
+     5,                         /*START_SERVERS*/
+     10,                        /*MAX_SERVERS*/
+     30,                        /*THREADS_PER_CHILD*/
+     30,                        /*MIN_SPARE_THREADS*/
+     60                        /*MAX_SPARE_THREADS*/
 };
 
 
@@ -65,11 +73,6 @@ int TIMEOUT = 300;
 int KEEPALIVE_TIMEOUT = 15;
 int MAX_KEEPALIVE_REQUESTS = 100;
 int MAX_SECS_TO_LINGER = 5;
-int START_CHILDS = 5;
-int MAX_CHILDS = 10;
-int START_SERVERS = 30;
-int MIN_FREE_SERVERS = 30;
-int MAX_FREE_SERVERS = 60;
 int MAX_REQUESTS_BEFORE_REALLOCATE_MEM = 100;
 int MAX_REQUESTS_PER_CHILD = 0;
 int DAEMON_MODE = 1;
@@ -96,33 +99,33 @@ extern int REMOTE_PROXY_USER_HEADER_ENCODED;
 
 
 /*Functions declaration */
-int parse_file(char *conf_file);
+int parse_file(const char *conf_file);
 
 /*config table functions*/
-int cfg_load_magicfile(char *directive, char **argv, void *setdata);
-int cfg_load_service(char *directive, char **argv, void *setdata);
-int cfg_service_alias(char *directive, char **argv, void *setdata);
-int cfg_load_module(char *directive, char **argv, void *setdata);
-int cfg_set_logformat(char *directive, char **argv, void *setdata);
-int cfg_set_logger(char *directive, char **argv, void *setdata);
-int cfg_set_accesslog(char *directive, char **argv, void *setdata);
-int cfg_set_debug_level(char *directive, char **argv, void *setdata);
-int cfg_set_debug_stdout(char *directive, char **argv, void *setdata);
-int cfg_set_body_maxmem(char *directive, char **argv, void *setdata);
-int cfg_set_tmp_dir(char *directive, char **argv, void *setdata);
-int cfg_set_acl_controllers(char *directive, char **argv, void *setdata);
-int cfg_set_auth_method(char *directive, char **argv, void *setdata);
-int cfg_include_config_file(char *directive, char **argv, void *setdata);
-int cfg_group_source_by_group(char *directive, char **argv, void *setdata);
-int cfg_group_source_by_user(char *directive, char **argv, void *setdata);
+int cfg_load_magicfile(const char *directive, const char **argv, void *setdata);
+int cfg_load_service(const char *directive, const char **argv, void *setdata);
+int cfg_service_alias(const char *directive, const char **argv, void *setdata);
+int cfg_load_module(const char *directive, const char **argv, void *setdata);
+int cfg_set_logformat(const char *directive, const char **argv, void *setdata);
+int cfg_set_logger(const char *directive, const char **argv, void *setdata);
+int cfg_set_accesslog(const char *directive, const char **argv, void *setdata);
+int cfg_set_debug_level(const char *directive, const char **argv, void *setdata);
+int cfg_set_debug_stdout(const char *directive, const char **argv, void *setdata);
+int cfg_set_body_maxmem(const char *directive, const char **argv, void *setdata);
+int cfg_set_tmp_dir(const char *directive, const char **argv, void *setdata);
+int cfg_set_acl_controllers(const char *directive, const char **argv, void *setdata);
+int cfg_set_auth_method(const char *directive, const char **argv, void *setdata);
+int cfg_include_config_file(const char *directive, const char **argv, void *setdata);
+int cfg_group_source_by_group(const char *directive, const char **argv, void *setdata);
+int cfg_group_source_by_user(const char *directive, const char **argv, void *setdata);
 
 /*The following 2 functions defined in access.c file*/
-int cfg_acl_add(char *directive, char **argv, void *setdata);
-int cfg_default_acl_access(char *directive, char **argv, void *setdata);
+int cfg_acl_add(const char *directive, const char **argv, void *setdata);
+int cfg_default_acl_access(const char *directive, const char **argv, void *setdata);
 /****/
 
 struct sub_table {
-     char *name;
+     const char *name;
      int type;
      struct ci_conf_entry *conf_table;
 };
@@ -135,11 +138,11 @@ static struct ci_conf_entry conf_variables[] = {
      {"KeepAlive", NULL, NULL, NULL},
      {"MaxKeepAliveRequests", &MAX_KEEPALIVE_REQUESTS, intl_cfg_set_int, NULL},
      {"KeepAliveTimeout", &KEEPALIVE_TIMEOUT, intl_cfg_set_int, NULL},
-     {"StartServers", &START_CHILDS, intl_cfg_set_int, NULL},
-     {"MaxServers", &MAX_CHILDS, intl_cfg_set_int, NULL},
-     {"MinSpareThreads", &MIN_FREE_SERVERS, intl_cfg_set_int, NULL},
-     {"MaxSpareThreads", &MAX_FREE_SERVERS, intl_cfg_set_int, NULL},
-     {"ThreadsPerChild", &START_SERVERS, intl_cfg_set_int, NULL},
+     {"StartServers", &CONF.START_SERVERS, intl_cfg_set_int, NULL},
+     {"MaxServers", &CONF.MAX_SERVERS, intl_cfg_set_int, NULL},
+     {"MinSpareThreads", &CONF.MIN_SPARE_THREADS, intl_cfg_set_int, NULL},
+     {"MaxSpareThreads", &CONF.MAX_SPARE_THREADS, intl_cfg_set_int, NULL},
+     {"ThreadsPerChild", &CONF.THREADS_PER_CHILD, intl_cfg_set_int, NULL},
      {"MaxRequestsPerChild", &MAX_REQUESTS_PER_CHILD, intl_cfg_set_int, NULL},
      {"MaxRequestsReallocateMem", &MAX_REQUESTS_BEFORE_REALLOCATE_MEM,
       intl_cfg_set_int, NULL},
@@ -211,7 +214,7 @@ void reset_conf_tables()
      conf_tables_num = 0;
 }
 
-int register_conf_table(char *name, struct ci_conf_entry *table, int type)
+int register_conf_table(const char *name, struct ci_conf_entry *table, int type)
 {
      struct sub_table *new;
      int i, insert_pos;
@@ -252,7 +255,7 @@ int register_conf_table(char *name, struct ci_conf_entry *table, int type)
      return 1;
 }
 
-struct ci_conf_entry *unregister_conf_table(char *name)
+struct ci_conf_entry *unregister_conf_table(const char *name)
 {
     int i;
     struct ci_conf_entry *table;
@@ -360,7 +363,7 @@ int print_variables()
    The following tree functions refered to non constant variables so
    the compilers in Win32 have problem to appeared in static arrays
 */
-int cfg_set_debug_level(char *directive, char **argv, void *setdata)
+int cfg_set_debug_level(const char *directive, const char **argv, void *setdata)
 {
     if (!DebugLevelSetFromCmd)
          return intl_cfg_set_int(directive, argv, &CI_DEBUG_LEVEL);
@@ -368,23 +371,23 @@ int cfg_set_debug_level(char *directive, char **argv, void *setdata)
     return 1;
 }
 
-int cfg_set_debug_level_cmd(char *directive, char **argv, void *setdata)
+int cfg_set_debug_level_cmd(const char *directive, const char **argv, void *setdata)
 {
      DebugLevelSetFromCmd = 1;
      return intl_cfg_set_int(directive, argv, &CI_DEBUG_LEVEL);
 }
 
-int cfg_set_debug_stdout(char *directive, char **argv, void *setdata)
+int cfg_set_debug_stdout(const char *directive, const char **argv, void *setdata)
 {
      return intl_cfg_enable(directive, argv, &CI_DEBUG_STDOUT);
 }
 
-int cfg_set_body_maxmem(char *directive, char **argv, void *setdata)
+int cfg_set_body_maxmem(const char *directive, const char **argv, void *setdata)
 {
      return intl_cfg_size_long(directive, argv, &CI_BODY_MAX_MEM);
 }
 
-int cfg_load_service(char *directive, char **argv, void *setdata)
+int cfg_load_service(const char *directive, const char **argv, void *setdata)
 {
      ci_service_module_t *service = NULL;
      if (argv == NULL || argv[0] == NULL || argv[1] == NULL) {
@@ -394,14 +397,14 @@ int cfg_load_service(char *directive, char **argv, void *setdata)
      ci_debug_printf(2, "Loading service :%s path %s\n", argv[0], argv[1]);
 
      if (!(service = register_service(argv[1]))) {
-          ci_debug_printf(1, "Error loading service\n");
+          ci_debug_printf(1, "Error loading service %s\n", argv[1]);
           return 0;
      }
      add_service_alias(argv[0], service->mod_name, NULL);
      return 1;
 }
 
-int cfg_service_alias(char *directive, char **argv, void *setdata)
+int cfg_service_alias(const char *directive, const char **argv, void *setdata)
 {
      char *service_args = NULL;
      if (argv == NULL || argv[0] == NULL || argv[1] == NULL) {
@@ -418,7 +421,7 @@ int cfg_service_alias(char *directive, char **argv, void *setdata)
      return 1;
 }
 
-int cfg_load_module(char *directive, char **argv, void *setdata)
+int cfg_load_module(const char *directive, const char **argv, void *setdata)
 {
      if (argv == NULL || argv[0] == NULL || argv[1] == NULL) {
           ci_debug_printf(1, "Missing arguments in LoadModule directive\n");
@@ -427,15 +430,15 @@ int cfg_load_module(char *directive, char **argv, void *setdata)
      ci_debug_printf(2, "Loading service :%s path %s\n", argv[0], argv[1]);
 
      if (!register_module(argv[1], argv[0])) {
-          ci_debug_printf(1, "Error loading service\n");
+          ci_debug_printf(1, "Error loading module %s, module path %s\n", argv[1], argv[0]);
           return 0;
      }
      return 1;
 }
 
-int cfg_load_magicfile(char *directive, char **argv, void *setdata)
+int cfg_load_magicfile(const char *directive, const char **argv, void *setdata)
 {
-     char *db_file;
+     const char *db_file;
      struct ci_magics_db *ndb;
      if (argv == NULL || argv[0] == NULL) {
           return 0;
@@ -443,7 +446,7 @@ int cfg_load_magicfile(char *directive, char **argv, void *setdata)
 
      db_file = argv[0];
      if (strcmp(CONF.magics_file, db_file) == 0) {
-         ci_debug_printf(2, "The db file %d is the same as default. Ignoring...\n", db_file); 
+         ci_debug_printf(2, "The db file %s is the same as default. Ignoring...\n", db_file); 
          return 1;
      }
      ci_debug_printf(2, "Going to load magic file %s\n", db_file);
@@ -458,8 +461,8 @@ int cfg_load_magicfile(char *directive, char **argv, void *setdata)
      return 1;
 }
 
-int logformat_add(char *name, char *format);
-int cfg_set_logformat(char *directive, char **argv, void *setdata)
+int logformat_add(const char *name, const char *format);
+int cfg_set_logformat(const char *directive, const char **argv, void *setdata)
 {
      if (argv == NULL || argv[0] == NULL || argv[1] == NULL) {
           ci_debug_printf(1, "Missing arguments in directive %s\n", directive);
@@ -469,10 +472,10 @@ int cfg_set_logformat(char *directive, char **argv, void *setdata)
      return logformat_add(argv[0], argv[1]);
 }
 
-int file_log_addlogfile(char *file, char *format, char **acls);
-int cfg_set_accesslog(char *directive, char **argv, void *setdata)
+int file_log_addlogfile(const char *file, const char *format, const char **acls);
+int cfg_set_accesslog(const char *directive, const char **argv, void *setdata)
 {
-     char **acls = NULL;
+     const char **acls = NULL;
      if (argv == NULL || argv[0] == NULL ) {
           ci_debug_printf(1, "Missing arguments in directive %s\n", directive);
           return 0;
@@ -486,7 +489,7 @@ int cfg_set_accesslog(char *directive, char **argv, void *setdata)
 
 
 extern logger_module_t file_logger;
-int cfg_set_logger(char *directive, char **argv, void *setdata)
+int cfg_set_logger(const char *directive, const char **argv, void *setdata)
 {
      logger_module_t *logger;
      if (argv == NULL || argv[0] == NULL) {
@@ -501,7 +504,7 @@ int cfg_set_logger(char *directive, char **argv, void *setdata)
      return 1;
 }
 
-int cfg_set_tmp_dir(char *directive, char **argv, void *setdata)
+int cfg_set_tmp_dir(const char *directive, const char **argv, void *setdata)
 {
      int len;
      if (argv == NULL || argv[0] == NULL) {
@@ -530,7 +533,7 @@ int cfg_set_tmp_dir(char *directive, char **argv, void *setdata)
      return 1;
 }
 
-int cfg_set_acl_controllers(char *directive, char **argv, void *setdata)
+int cfg_set_acl_controllers(const char *directive, const char **argv, void *setdata)
 {
      int i, k, argc, ret;
      access_control_module_t *acl_mod;
@@ -564,9 +567,9 @@ int cfg_set_acl_controllers(char *directive, char **argv, void *setdata)
 }
 
 
-int cfg_set_auth_method(char *directive, char **argv, void *setdata)
+int cfg_set_auth_method(const char *directive, const char **argv, void *setdata)
 {
-     char *method = NULL;
+     const char *method = NULL;
      if (argv == NULL || argv[0] == NULL || argv[1] == NULL) {
           return 0;
      }
@@ -575,12 +578,32 @@ int cfg_set_auth_method(char *directive, char **argv, void *setdata)
      if (strncasecmp(argv[1], "none", 4) == 0) {
           return set_method_authenticators(method, NULL);
      }
-     return set_method_authenticators(method, argv + 1);
+     return set_method_authenticators(method, (const char **)argv + 1);
 }
 
-int cfg_include_config_file(char *directive, char **argv, void *setdata)
+int cfg_acl_add(const char *directive, const char **argv, void *setdata)
 {
-     char path[CI_MAX_PATH], *cfg_file;
+    const char *acl_name, *acl_type;
+    int argc, ok;
+ 
+    if (!argv[0] || !argv[1] || !argv[2]) /* at least an argument */
+        return 0;
+     
+     
+    acl_name = argv[0];
+    acl_type = argv[1];
+    for (argc = 2, ok =1; argv[argc] != NULL && ok; argc++){
+        ci_debug_printf(2, "Adding to acl %s the data %s\n", acl_name, argv[argc]);
+        ok = ci_acl_add_data(acl_name, acl_type, argv[argc]);
+    }
+    ci_debug_printf(2, "New ACL with name:%s and  ACL Type: %s\n", argv[0], argv[1]);
+    return ok;
+}
+
+int cfg_include_config_file(const char *directive, const char **argv, void *setdata)
+{
+    char path[CI_MAX_PATH];
+    const char *cfg_file;
 
      if (argv == NULL || argv[0] == NULL) {
           return 0;
@@ -599,24 +622,24 @@ int cfg_include_config_file(char *directive, char **argv, void *setdata)
 int group_source_add_by_group(const char *table_name);
 int group_source_add_by_user(const char *table_name);
 
-int cfg_group_source_by_group(char *directive, char **argv, void *setdata)
+int cfg_group_source_by_group(const char *directive, const char **argv, void *setdata)
 {
-  char *group_table = NULL;
+  const char *group_table = NULL;
   if (argv == NULL || argv[0] == NULL) {
     return 0;
   }
   group_table = argv[0];
-  return group_source_add_by_group((const char *)group_table);
+  return group_source_add_by_group(group_table);
 }
 
-int cfg_group_source_by_user(char *directive, char **argv, void *setdata)
+int cfg_group_source_by_user(const char *directive, const char **argv, void *setdata)
 {
-  char *group_table = NULL;
+  const char *group_table = NULL;
   if (argv == NULL || argv[0] == NULL) {
     return 0;
   }
   group_table = argv[0];
-  return group_source_add_by_user((const char *)group_table);
+  return group_source_add_by_user(group_table);
 }
 
 /**************************************************************************/
@@ -670,8 +693,8 @@ struct ci_conf_entry *find_action(char *str, char **arg)
 
 char **split_args(char *args)
 {
-     int len, i = 0;
-     char **argv = NULL, *str, *end;
+    int len, i = 0, brkt;
+    char **argv = NULL, *str, *end, *p;
      argv = malloc((MAX_ARGS + 1) * sizeof(char *));
      end = args;
      do {
@@ -679,12 +702,23 @@ char **split_args(char *args)
           if (*end == '"') {
                end++;
                str = end;
-               while (*end != '\0' && *end != '"')
-                    end++;
+               while (*end != '\0' && *end != '"') {
+                   /*support escaped \" */
+                   if (*end == '\\' && *(end+1) == '"') {
+                       for (p = end; *p != '\0'; p++) 
+                           *p = *(p+1); 
+                   }
+                   end++;
+               }
           }
           else {
-               while (*end != '\0' && !isspace(*end))
+              /*Support arguments in the form arg{a, b...}*/
+              brkt = 0;
+              while (*end != '\0' && (!isspace(*end) || brkt)) {
+                    if (*end == '{') brkt = 1;
+                    else if (brkt && *end == '}') brkt = 0;
                     end++;
+              }
           }
           len = end - str;
 
@@ -741,7 +775,7 @@ int process_line(char *orig_line)
 
      if (entry && entry->action) {
           argv = split_args(args);
-          if ((*(entry->action)) (entry->name, argv, entry->data) == 0)
+          if ((*(entry->action)) (entry->name, (const char **)argv, entry->data) == 0)
 	      ret = 0;
           free_args(argv);
           return ret;
@@ -754,14 +788,15 @@ int process_line(char *orig_line)
 
 static int PARSE_LEVEL = 0;
 
-int parse_file(char *conf_file)
+int parse_file(const char *conf_file)
 {
      FILE *f_conf;
      char line[LINESIZE];
      int line_count, ret_value;
 
-     if (PARSE_LEVEL >= 3) {
-          ci_debug_printf(1, "Parse level >3. I will not parse file:%s\n",
+     if (PARSE_LEVEL >= MAX_INCLUDE_LEVEL) {
+          ci_debug_printf(1, "Include level > %d. I will not parse file:%s\n",
+                          MAX_INCLUDE_LEVEL,
                           conf_file);
           return 0;
      }
@@ -857,14 +892,20 @@ void release_modules();
 void ci_dlib_closeall();
 int log_open();
 void ci_magic_db_free();
+void reset_http_auth();
 
 void system_shutdown()
 {
     /*
       - reset commands table
     */
+    commands_reset();
+
+    /*
+     - clean registry
+    */
+    ci_registry_clean();
 
-    reset_commands();
     /*
       - close/release services and modules
     */
@@ -1016,7 +1057,7 @@ void cfg_default_value_restore_all()
 /********************************************************************/
 /* functions for setting parameters and saving the default values   */
 
-int intl_cfg_set_str(char *directive, char **argv, void *setdata)
+int intl_cfg_set_str(const char *directive, const char **argv, void *setdata)
 {
      if (!setdata)
           return 0;
@@ -1025,7 +1066,7 @@ int intl_cfg_set_str(char *directive, char **argv, void *setdata)
      return ci_cfg_set_str(directive, argv, setdata);
 }
 
-int intl_cfg_set_int(char *directive, char **argv, void *setdata)
+int intl_cfg_set_int(const char *directive, const char **argv, void *setdata)
 {
      if (!setdata)
           return 0;
@@ -1033,7 +1074,7 @@ int intl_cfg_set_int(char *directive, char **argv, void *setdata)
      return ci_cfg_set_int(directive, argv, setdata);
 }
 
-int intl_cfg_onoff(char *directive, char **argv, void *setdata)
+int intl_cfg_onoff(const char *directive, const char **argv, void *setdata)
 {
      if (!setdata)
           return 0;
@@ -1041,7 +1082,7 @@ int intl_cfg_onoff(char *directive, char **argv, void *setdata)
      return ci_cfg_onoff(directive, argv, setdata);
 }
 
-int intl_cfg_disable(char *directive, char **argv, void *setdata)
+int intl_cfg_disable(const char *directive, const char **argv, void *setdata)
 {
      if (!setdata)
           return 0;
@@ -1049,7 +1090,7 @@ int intl_cfg_disable(char *directive, char **argv, void *setdata)
      return ci_cfg_disable(directive, argv, setdata);
 }
 
-int intl_cfg_enable(char *directive, char **argv, void *setdata)
+int intl_cfg_enable(const char *directive, const char **argv, void *setdata)
 {
      if (!setdata)
           return 0;
@@ -1057,7 +1098,7 @@ int intl_cfg_enable(char *directive, char **argv, void *setdata)
      return ci_cfg_enable(directive, argv, setdata);
 }
 
-int intl_cfg_size_off(char *directive, char **argv, void *setdata)
+int intl_cfg_size_off(const char *directive, const char **argv, void *setdata)
 {
      if (!setdata)
           return 0;
@@ -1065,7 +1106,7 @@ int intl_cfg_size_off(char *directive, char **argv, void *setdata)
      return ci_cfg_size_off(directive, argv, setdata);
 }
 
-int intl_cfg_size_long(char *directive, char **argv, void *setdata)
+int intl_cfg_size_long(const char *directive, const char **argv, void *setdata)
 {
      if (!setdata)
           return 0;
