@@ -88,6 +88,7 @@ const struct ci_error_code ci_error_codes[] = {
      {100, "Continue"},         /*Continue after ICAP Preview */
      {200, "OK"},
      {204, "Unmodified"},       /*No modifications needed */
+     {206, "Partial Content"},  /*Partial content modification*/
      {400, "Bad request"},      /*Bad request */
      {401, "Unauthorized"},
      {403, "Forbidden"},
@@ -112,13 +113,13 @@ const struct ci_error_code ci_error_codes[] = {
 /*
 #ifdef __CYGWIN__
 int ci_error_code(int ec){
-     return (ec>=EC_100&&ec<=EC_500?CI_ErrorCodes[ec].code:1000);
+     return (ec>=EC_100&&ec<EC_MAX?ci_error_codes[ec].code:1000);
 }
 
 const char *unknownerrorcode="UNKNOWN ERROR CODE";
 
 const char *ci_error_code_string(int ec){
-     return (ec>=EC_100&&ec<=EC_505?CI_ErrorCodes[ec].str:unknownerrorcode);
+     return (ec>=EC_100&&ec<EC_MAX?ci_error_codes[ec].str:unknownerrorcode);
 }
 #endif
 */
@@ -156,9 +157,12 @@ ci_headers_list_t *ci_headers_create()
 {
      ci_headers_list_t *h;
      h = malloc(sizeof(ci_headers_list_t));
-     if (!h)
+     if (!h) {
+          ci_debug_printf(1, "Error allocation memory for ci_headers_list_t (header.c: ci_headers_create)\n");
           return NULL;
-
+     }
+     h->headers = NULL;
+     h->buf = NULL;
      if (!(h->headers = malloc(HEADERSTARTSIZE * sizeof(char *)))
          || !(h->buf = malloc(HEADSBUFSIZE * sizeof(char)))) {
           ci_debug_printf(1, "Server Error: Error allocation memory \n");
@@ -166,8 +170,7 @@ ci_headers_list_t *ci_headers_create()
                free(h->headers);
           if (h->buf)
                free(h->buf);
-          if (h)
-               free(h);
+          free(h);
           return NULL;
      }
 
@@ -214,7 +217,7 @@ void ci_headers_reset(ci_headers_list_t * h)
      h->bufused = 0;
 }
 
-char *ci_headers_add(ci_headers_list_t * h, char *line)
+const char *ci_headers_add(ci_headers_list_t * h, const char *line)
 {
      char *newhead, **newspace, *newbuf;
      int len, linelen;
@@ -261,7 +264,7 @@ char *ci_headers_add(ci_headers_list_t * h, char *line)
 }
 
 
-int ci_headers_addheaders(ci_headers_list_t * h, ci_headers_list_t * headers)
+int ci_headers_addheaders(ci_headers_list_t * h, const ci_headers_list_t * headers)
 {
      int remains, len, i;
      char *newbuf, **newspace;
@@ -308,7 +311,7 @@ int ci_headers_addheaders(ci_headers_list_t * h, ci_headers_list_t * headers)
 }
 
 
-char *ci_headers_search(ci_headers_list_t * h, char *header)
+const char *ci_headers_search(ci_headers_list_t * h, const char *header)
 {
      int i;
      for (i = 0; i < h->used; i++) {
@@ -318,9 +321,9 @@ char *ci_headers_search(ci_headers_list_t * h, char *header)
      return NULL;
 }
 
-char *ci_headers_value(ci_headers_list_t * h, char *header)
+const char *ci_headers_value(ci_headers_list_t * h, const char *header)
 {
-     char *phead;
+     const char *phead;
      if (!(phead = ci_headers_search(h, header)))
           return NULL;
      while (*phead != '\0' && *phead != ':')
@@ -333,7 +336,33 @@ char *ci_headers_value(ci_headers_list_t * h, char *header)
      return phead;
 }
 
-int ci_headers_remove(ci_headers_list_t * h, char *header)
+const char *ci_headers_copy_value(ci_headers_list_t * h, const char *header, char *buf, size_t len)
+{
+     const char *phead;
+     int i;
+     if (!(phead = ci_headers_search(h, header)))
+          return NULL;
+     if (h->packed)
+         while (*phead != '\0' && *phead != ':' && *phead != '\r' && *phead !='\n') phead++;
+     else
+         while (*phead != '\0' && *phead != ':') phead++;
+
+     if (*phead != ':')
+          return NULL;
+     phead++;
+     /*skip spaces at the beginning*/
+     while (isspace(*phead) && *phead != '\0')
+          phead++;
+
+     /*phead now points to the biggining of the header value.*/
+     /*copy value to buf*/
+     for(i=0; i < len-1 && *phead != '\0' && *phead != '\r' && *phead != '\n'; i++, phead++)
+         buf[i] = *phead;
+     buf[i] = '\0';
+     return buf;
+}
+
+int ci_headers_remove(ci_headers_list_t * h, const char *header)
 {
      char *phead;
      int i, j, header_len, rest_len;
@@ -378,7 +407,7 @@ int ci_headers_remove(ci_headers_list_t * h, char *header)
      return 0;
 }
 
-char *ci_headers_replace(ci_headers_list_t * h, char *header, char *newval)
+const char *ci_headers_replace(ci_headers_list_t * h, const char *header, const char *newval)
 {
     if (h->packed) /*Not in edit mode*/
 	 return NULL;
@@ -386,6 +415,31 @@ char *ci_headers_replace(ci_headers_list_t * h, char *header, char *newval)
      return NULL;
 }
 
+#define eoh(s) ((*s == '\r' && *(s+1) == '\n' && *(s+2) != '\t' && *(s+2) != ' ') || (*s == '\n' && *(s+1) != '\t' && *(s+1) != ' '))
+
+int ci_headers_iterate(ci_headers_list_t * h, void *data, void (*fn)(void *, const char  *head, const char  *value))
+{
+    char header[256];
+    char value[8124];
+    char *s;
+    int i, j;
+    for (i = 0; i < h->used; i++) {
+        s = h->headers[i];
+        for (j = 0;  j < sizeof(header)-1 && *s != ':' &&  *s != '\0' && *s != '\r' && *s!='\n'; s++, j++)
+            header[j] = *s;
+        header[j] = '\0';
+        j = 0;
+        if (*s == ':') {
+            s++;
+            while (*s == ' ') s++;
+            for (j = 0;  j < sizeof(value)-1 &&  *s != '\0' && !eoh(s); s++, j++)
+                value[j] = *s;
+        }
+        value[j] = '\0';
+        fn(data, header, value);
+    }
+    return 1;
+}
 
 void ci_headers_pack(ci_headers_list_t * h)
 {                               /*Put the \r\n sequence at the end of each header before sending...... */
@@ -410,7 +464,7 @@ void ci_headers_pack(ci_headers_list_t * h)
           h->buf[h->bufused] = '\n';
           h->bufused++;
      }
-     h->packed = 0;
+     h->packed = 1;
 }
 
 
@@ -509,7 +563,7 @@ void destroy_encaps_entity(ci_encaps_entity_t * e)
      free(e);
 }
 
-int get_encaps_type(char *buf, int *val, char **endpoint)
+int get_encaps_type(const char *buf, int *val, char **endpoint)
 {
 
      if (0 == strncmp(buf, "req-hdr", 7)) {
