@@ -67,14 +67,24 @@
 
 # if defined (CI_BUILD_MODULE)
 #   define CI_DECLARE_MOD_DATA   __declspec(dllexport)
+#   define CI_DECLARE_MOD_FUNC(type)   __declspec(dllexport) type
 # endif
 
 #else
 
+/*
+  
+*/
+#if defined (USE_VISIBILITY_ATTRIBUTE)
+#define CI_DECLARE_FUNC(type) __attribute__ ((visibility ("default"))) type
+#define CI_DECLARE_DATA __attribute__ ((visibility ("default")))
+#define CI_DECLARE_MOD_DATA __attribute__ ((visibility ("default")))
+#else
 #define CI_DECLARE_FUNC(type) type
 #define CI_DECLARE_DATA
 #define CI_DECLARE_MOD_DATA
 #endif
+#endif
 
 /*
   Here we are define the ci_off_t type to support large files
@@ -110,11 +120,13 @@ typedef off_t ci_off_t;
 #endif
 
 /*
-  Use 8 bytes alignment.
-  TODO: The correct alignment size should be detected in configure.in 
-  and defined in c-icap-conf.h file
- */
-#define _CI_ALIGN(val) ((val+7)&~7)
+  Detect n-bytes alignment.
+  Old 8 bytes alignment macro:
+  #define _CI_ALIGN(val) ((val+7)&~7)
+*/
+struct _ci_align_test {char n[1]; double d;};
+#define _CI_NBYTES_ALIGNMENT ((size_t) &(((struct _ci_align_test *)0)[0].d))
+#define _CI_ALIGN(val) ((val+(_CI_NBYTES_ALIGNMENT - 1))&~(_CI_NBYTES_ALIGNMENT - 1))
 
 #define ICAP_OPTIONS   0x01
 #define ICAP_REQMOD    0x02
@@ -126,11 +138,13 @@ CI_DECLARE_DATA extern const char *ci_methods[];
 #define ci_method_string(method) (method<=ICAP_RESPMOD && method>=ICAP_OPTIONS ?ci_methods[method]:"UNKNOWN")
 
 
-enum ci_error_codes { EC_100, EC_200, EC_204, EC_400, 
+enum ci_error_codes { EC_100, EC_200, EC_204, EC_206, EC_400, 
                    EC_401, EC_403,
 		   EC_404, EC_405, EC_407, EC_408,
 		   EC_500, EC_501, EC_502, 
-		   EC_503, EC_505};
+		   EC_503, EC_505,
+                   EC_MAX
+};
 
 typedef struct ci_error_code{
   int code;
@@ -138,8 +152,8 @@ typedef struct ci_error_code{
 } ci_error_code_t;
 
 CI_DECLARE_DATA extern const struct ci_error_code ci_error_codes[];
-#define ci_error_code(ec) (ec>=EC_100&&ec<=EC_500?ci_error_codes[ec].code:1000)
-#define ci_error_code_string(ec) (ec>=EC_100&&ec<=EC_500?ci_error_codes[ec].str:"UNKNOWN ERROR CODE")
+#define ci_error_code(ec) (ec>=EC_100&&ec<EC_MAX?ci_error_codes[ec].code:1000)
+#define ci_error_code_string(ec) (ec>=EC_100&&ec<EC_MAX?ci_error_codes[ec].str:"UNKNOWN ERROR CODE")
 
 
 #define ICAP_EOL "\r\n"
