@@ -23,6 +23,10 @@
 
 #include "request.h"
 
+#ifdef __cplusplus
+extern "C"
+{
+#endif
 
 int log_open();
 void log_close();
@@ -33,4 +37,10 @@ void log_access(ci_request_t *req,int status);
 void log_server(ci_request_t *req, const char *format, ... );
 void vlog_server(ci_request_t *req, const char *format, va_list ap);
 
+/* The followings can be used by modules */
+CI_DECLARE_FUNC(char *) logformat_fmt(const char *name);
+#ifdef __cplusplus
+}
+#endif
+
 #endif
