@@ -22,7 +22,6 @@
 #define __NET_IO_H
 
 #ifndef _WIN32
-
 #include <netinet/in.h>
 #include <sys/ioctl.h>
 #include <sys/select.h>
@@ -31,19 +30,23 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 #include <fcntl.h>
-
-
-#define ci_socket int
-#define CI_SOCKET_ERROR -1
-
-
 #else
 #include <WinSock2.h>
+#endif
+
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+
+#ifndef _WIN32
+#define ci_socket int
+#define CI_SOCKET_ERROR -1
+#else
 #define ci_socket SOCKET
 #define CI_SOCKET_ERROR INVALID_SOCKET
 #endif
 
-
 typedef struct ci_sockaddr{
 #ifdef USE_IPV6
      struct sockaddr_storage  sockaddr;
@@ -130,7 +133,7 @@ CI_DECLARE_FUNC(void) ci_sockaddr_set_port(ci_sockaddr_t *addr, int port);
 #endif
 
 CI_DECLARE_FUNC(const char *) ci_sockaddr_t_to_host(ci_sockaddr_t *addr, char *hname, int maxhostlen);
-CI_DECLARE_FUNC(int) ci_host_to_sockaddr_t(char *servername, ci_sockaddr_t * addr, int proto);
+CI_DECLARE_FUNC(int) ci_host_to_sockaddr_t(const char *servername, ci_sockaddr_t * addr, int proto);
 
 CI_DECLARE_FUNC(void) ci_copy_connection(ci_connection_t *dest, ci_connection_t *src);
 
@@ -152,4 +155,8 @@ CI_DECLARE_FUNC(int) ci_write_nonblock(ci_socket fd, const void *buf,size_t coun
 CI_DECLARE_FUNC(int) ci_linger_close(ci_socket fd,int secs_to_linger);
 CI_DECLARE_FUNC(int) ci_hard_close(ci_socket fd);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
