@@ -62,10 +62,16 @@ struct info_req_data {
      int *child_pids;
      int free_servers;
      int used_servers;
+     unsigned int closing_childs;
+     int *closing_child_pids;
+     unsigned int started_childs;
+     unsigned int closed_childs;
+     unsigned int crashed_childs;
      struct stat_memblock *collect_stats;
 };
 
 extern struct childs_queue *childs_queue;
+extern ci_proc_mutex_t accept_mutex;
 
 int build_statistics(struct info_req_data *info_data);
 
@@ -92,6 +98,11 @@ void *info_init_request_data(ci_request_t * req)
      info_data->child_pids = malloc(childs_queue->size * sizeof(int));
      info_data->free_servers = 0;
      info_data->used_servers = 0;
+     info_data->closing_childs = 0;
+     info_data->closing_child_pids = malloc(childs_queue->size * sizeof(int));
+     info_data->started_childs = 0;
+     info_data->closed_childs = 0;
+     info_data->crashed_childs = 0;
      info_data->txt_mode = 0;
      if (req->args) {
           if (strstr(req->args, "view=text"))
@@ -174,7 +185,7 @@ void fill_queue_statistics(struct childs_queue *q, struct info_req_data *info_da
     int i;
     int requests = 0;
     struct stat_memblock *stats, copy_stats;
-
+    struct server_statistics *srv_stats;
      if (!q->childs)
           return;
 
@@ -196,8 +207,11 @@ void fill_queue_statistics(struct childs_queue *q, struct info_req_data *info_da
 		 + stats->counters64_size*sizeof(uint64_t);
 
 	       ci_stat_memblock_merge(info_data->collect_stats, &copy_stats);
-
-
+          }
+          else if (q->childs[i].pid != 0 && q->childs[i].to_be_killed) {
+               if (info_data->closing_child_pids)
+                   info_data->closing_child_pids[info_data->closing_childs] = q->childs[i].pid;
+               info_data->closing_childs++;
           }
      }
      /*Merge history data*/
@@ -209,6 +223,13 @@ void fill_queue_statistics(struct childs_queue *q, struct info_req_data *info_da
        + stats->counters64_size*sizeof(uint64_t);
 
      ci_stat_memblock_merge(info_data->collect_stats, &copy_stats);     
+
+     srv_stats = 
+         (struct server_statistics *)(q->stats_area + q->size * q->stats_block_size + q->stats_block_size);
+     /*Compute server statistics*/
+     info_data->started_childs = srv_stats->started_childs;
+     info_data->closed_childs = srv_stats->closed_childs;
+     info_data->crashed_childs = srv_stats->crashed_childs;
 }
 
 struct stats_tmpl {
@@ -218,34 +239,50 @@ struct stats_tmpl {
    char *childsHeader;
    char *childs_tmpl;
    char *childsEnd;
+   char *closingChildsHeader;
+   char *semaphores_tmpl;
+   char *sharedMem_tmpl;
    char *statline_tmpl_int;
    char *statline_tmpl_kbs;
 };
 
 struct stats_tmpl txt_tmpl = {
   "Running Servers Statistics\n===========================\n"\
-    "Childs number: %d\nFree Servers:%d\nUsed Servers:%d\n\n\n",
+  "Children number: %d\nFree Servers: %d\nUsed Servers: %d\n"\
+  "Started Processes: %u\nClosed Processes: %u\nCrashed Processes: %u\n"\
+  "Closing Processes: %u"\
+  "\n\n",
   "\n%s Statistics\n==================\n",
   "",
   "Child pids:",
   " %d",
   "\n",
+  "Closing children pids:",
+  "%s semaphores in use: %s\n",
+  "%s shared mem in use: %s\n",
   "%s : %lld\n",
   "%s : %lld Kbs %d bytes\n"
 };
 
 struct stats_tmpl html_tmpl = {
   "<H1>Running Servers Statistics</H1>\n"\
-    "<TABLE>"\
-    "<TR><TH>Childs number:</TH><TD> %d<TD>"\
-    "<TR><TH>Free Servers:</TH><TD> %d<TD>"\
-    "<TR><TH>Used Servers:</TH><TD> %d<TD>"\
-    "</TABLE>\n",
+  "<TABLE>"                                     \
+  "<TR><TH>Children number:</TH><TD> %d<TD>"                     \
+  "<TR><TH>Free Servers:</TH><TD> %d<TD>"                      \
+  "<TR><TH>Used Servers:</TH><TD> %d<TD>"                      \
+  "<TR><TH>Started Processes :</TH><TD> %u<TD>"                \
+  "<TR><TH>Closed Processes: </TH><TD>%u<TD>"                  \
+  "<TR><TH>Crashed Processes: </TH><TD>%u<TD>"                 \
+  "<TR><TH>Closing Processes: </TH><TD>%u<TD>"                 \
+  "</TABLE>\n",
   "<H1>%s Statistics</H1>\n<TABLE>",
   "</TABLE>",
   "<TABLE> <TR><TH>Child pids:</TH>",
   "<TD> %d</TD>",
   "</TR></TABLE>\n",
+  "<TABLE> <TR><TH>Closing children pids:</TH>",
+  "<TR><TH>%s semaphores in use:</TH><TD>%s</TD>\n",
+  "<TR><TH>%s shared mem in use:</TH><TD>%s</TD>\n",
   "<TR><TH>%s:</TH><TD>  %lld</TD>\n",
   "<TR><TH>%s:</TH><TD>  %lld Kbs %d bytes</TD>\n"
 };
@@ -254,6 +291,7 @@ struct stats_tmpl html_tmpl = {
 int build_statistics(struct info_req_data *info_data)
 {    
      char buf[LOCAL_BUF_SIZE];
+     char buf2[LOCAL_BUF_SIZE];
      int sz, gid, k;
      char *stat_group;
      struct stats_tmpl *tmpl;
@@ -271,23 +309,66 @@ int build_statistics(struct info_req_data *info_data)
      sz = snprintf(buf, LOCAL_BUF_SIZE,tmpl->gen_template,
 		   info_data->childs,
 		   info_data->free_servers,
-		   info_data->used_servers);
+		   info_data->used_servers,
+                   info_data->started_childs,
+                   info_data->closed_childs,
+                   info_data->crashed_childs,
+                   info_data->closing_childs
+         );
 
-     if (sz > LOCAL_BUF_SIZE)
+     if (sz > LOCAL_BUF_SIZE) 
          sz = LOCAL_BUF_SIZE;
-    
+
      ci_membuf_write(info_data->body,buf, sz, 0);
 
      /*print childs pids ...*/
      ci_membuf_write(info_data->body, tmpl->childsHeader, strlen(tmpl->childsHeader), 0);
      for (k =0; k < info_data->childs; k++) {
           sz = snprintf(buf, LOCAL_BUF_SIZE, tmpl->childs_tmpl, info_data->child_pids[k]);
-          if (sz > LOCAL_BUF_SIZE)
+          if (sz > LOCAL_BUF_SIZE) 
               sz = LOCAL_BUF_SIZE;
           ci_membuf_write(info_data->body,buf, sz, 0);
      } 
      ci_membuf_write(info_data->body, tmpl->childsEnd, strlen(tmpl->childsEnd), 0);
 
+     /*print closing childs pids ...*/
+     ci_membuf_write(info_data->body, tmpl->closingChildsHeader, strlen(tmpl->closingChildsHeader), 0);
+     for (k =0; k < info_data->closing_childs; k++) {
+          sz = snprintf(buf, LOCAL_BUF_SIZE, tmpl->childs_tmpl, info_data->closing_child_pids[k]);
+          if (sz > LOCAL_BUF_SIZE) 
+              sz = LOCAL_BUF_SIZE;
+          ci_membuf_write(info_data->body,buf, sz, 0);
+     } 
+     ci_membuf_write(info_data->body, tmpl->childsEnd, strlen(tmpl->childsEnd), 0);
+
+     /*Print semaphores*/ /*Print shared mem info*/
+#if defined(USE_SYSV_IPC_MUTEX)
+     snprintf(buf2, LOCAL_BUF_SIZE, "%d %d", accept_mutex, childs_queue->queue_mtx);
+     sz = snprintf(buf, LOCAL_BUF_SIZE, tmpl->semaphores_tmpl, "IPC", buf2);
+#elif defined(USE_POSIX_SEMAPHORES)
+     snprintf(buf2, LOCAL_BUF_SIZE, "%p %p", &accept_mutex, &childs_queue->queue_mtx);
+     sz = snprintf(buf, LOCAL_BUF_SIZE, tmpl->semaphores_tmpl, "POSIX", buf2);
+#elif defined(USE_POSIX_FILE_LOCK)
+     snprintf(buf2, LOCAL_BUF_SIZE, "%s %s", accept_mutex.filename, &childs_queue->queue_mtx.filename);
+     sz = snprintf(buf, LOCAL_BUF_SIZE, tmpl->semaphores_tmpl, "Lockfile", buf2);
+#else
+     sz = snprintf(buf, LOCAL_BUF_SIZE, tmpl->sharedMem_tmpl, "Unknown", "Unknown");
+#endif
+     ci_membuf_write(info_data->body,buf, sz, 0);
+
+/*Print shared mem*/
+#if defined(USE_SYSV_IPC)
+     snprintf(buf2, LOCAL_BUF_SIZE, "%d %d kbs", childs_queue->shmid, (childs_queue->shared_mem_size/1024));
+     sz = snprintf(buf, LOCAL_BUF_SIZE, tmpl->sharedMem_tmpl, "IPC", buf2);
+#elif defined(USE_POSIX_MAPPED_FILES)
+     snprintf(buf2, LOCAL_BUF_SIZE, "%p %d Kbs", childs_queue->shmid.mem, (childs_queue->shmid.size/1024));
+     sz = snprintf(buf, LOCAL_BUF_SIZE, tmpl->sharedMem_tmpl, "MMAP", buf2);
+#else
+     sz = snprintf(buf, LOCAL_BUF_SIZE, tmpl->sharedMem_tmpl, "Unknown", "Unknown");
+#endif
+     ci_membuf_write(info_data->body,buf, sz, 0);
+
+
      for (gid = 0; gid < STAT_GROUPS.entries_num; gid++) {
           stat_group = STAT_GROUPS.groups[gid];
 
