@@ -36,6 +36,7 @@ struct ci_lookup_table_type file_table_type={
     file_table_close,
     file_table_search,
     file_table_release_result,
+    NULL,
     "file"
 };
 
@@ -59,7 +60,7 @@ struct text_table_entry *alloc_text_table_entry (int val_num, ci_mem_allocator_t
     e->next = NULL;
     if(!e) {
         ci_debug_printf(1,"Error allocating memory for table entry \n");
-        return 0;
+        return NULL;
     }
     if (val_num>0) {
         e->vals = allocator->alloc(allocator, (val_num + 1)*sizeof(void *));
@@ -67,7 +68,7 @@ struct text_table_entry *alloc_text_table_entry (int val_num, ci_mem_allocator_t
             allocator->free(allocator, e);
             e=NULL;
             ci_debug_printf(1,"Error allocating memory for values of  table entry.\n");
-            return 0;
+            return NULL;
         }
         for(i=0; i< val_num + 1; i++) 
             e->vals[i] = NULL;
@@ -150,14 +151,15 @@ int read_row(FILE *f, int cols, struct text_table_entry **e,
      s=line;
      while(*s==' ' || *s == '\t') s++;
      val=s; 
-     
-     if( row_cols==1 ) 
-        end=s+strlen(s);
-     else
-        end=index(s,':');
+    
+     end = NULL;
+     if (row_cols > 1)
+         end=index(s,':');
+     if (end == NULL) /*no ":" char or the only column is the key (row_cols <=1)*/
+         end=s+strlen(s);
 
      s=end+1; /*Now points to the end (*s='\0') or after the ':' */
-      
+
      end--; 
      while(*end==' ' || *end=='\t') end--;
      *(end+1)='\0';
@@ -170,7 +172,7 @@ int read_row(FILE *f, int cols, struct text_table_entry **e,
          return -1;
      }
 
-     if(row_cols!=1) {
+     if(row_cols > 1) {
          assert((*e)->vals);
          for (i=0; *s!='\0'; i++) { /*probably we have vals*/
              if(i>=row_cols) {
@@ -205,11 +207,15 @@ int load_text_table(char *filename, struct ci_lookup_table *table) {
      struct text_table_entry *e, *l = NULL;
      int rows, ret;
      struct text_table *text_table = (struct text_table *)table->data;
-     if ((f = fopen(filename, "r+")) == NULL) {
+     if ((f = fopen(filename, "r")) == NULL) {
 	 ci_debug_printf(1, "Error opening file: %s\n", filename);
 	 return 0;
      }
+
      rows = 0;
+     if (text_table->entries != NULL) /*if table is not empty try to find the last element*/
+         for (l = text_table->entries; l->next != NULL; l = l->next); 
+
      while(0 < (ret=read_row(f, table->cols, &e, table))) {
          if (e) {
              e->next = NULL;
@@ -252,6 +258,7 @@ void *file_table_open(struct ci_lookup_table *table)
     return (table->data=NULL);
   }
   text_table->hash_table = NULL;
+  text_table->rows = 0;
   return text_table;
 }
 
@@ -325,6 +332,7 @@ struct ci_lookup_table_type hash_table_type={
     hash_table_close,
     hash_table_search,
     hash_table_release_result,
+    NULL,
     "hash"
 };
 
@@ -359,8 +367,8 @@ void  hash_table_close(struct ci_lookup_table *table)
     struct text_table *text_table = (struct text_table *)table->data;
     if (text_table && text_table->hash_table) {
         /*destroy the hash table */
-	ci_hash_destroy(text_table->hash_table);
-	text_table->hash_table = NULL;
+        ci_hash_destroy(text_table->hash_table);
+        text_table->hash_table = NULL;
     }
     /*... and then call the file_table_close:*/
     file_table_close(table);
@@ -368,7 +376,7 @@ void  hash_table_close(struct ci_lookup_table *table)
 
 void *hash_table_search(struct ci_lookup_table *table, void *key, void ***vals)
 {
-    struct text_table_entry *e;
+    const struct text_table_entry *e;
     struct text_table *text_table = (struct text_table *)table->data;
 
     if(!text_table) {
@@ -403,6 +411,7 @@ struct ci_lookup_table_type regex_table_type={
     regex_table_close,
     regex_table_search,
     regex_table_release_result,
+    NULL,
     "regex"
 };
 
