@@ -33,12 +33,14 @@ void *ldap_table_open(struct ci_lookup_table *table);
 void  ldap_table_close(struct ci_lookup_table *table);
 void *ldap_table_search(struct ci_lookup_table *table, void *key, void ***vals);
 void  ldap_table_release_result(struct ci_lookup_table *table_data,void **val);
+const void *ldap_table_get_row(struct ci_lookup_table *table, const void *key, const char *columns[], void ***vals);
 
 struct ci_lookup_table_type ldap_table_type={
     ldap_table_open,
     ldap_table_close,
     ldap_table_search,
     ldap_table_release_result,
+    NULL,
     "ldap"
 };
 
@@ -255,13 +257,13 @@ LDAP *ldap_connection_open(struct ldap_connections_pool *pool)
 
 
   conn=malloc(sizeof(struct ldap_connection));
-  conn->hits = 1;
   if (!conn) {
 #ifdef LDAP_MAX_CONNECTIONS
      ci_thread_mutex_unlock(&pool->mutex);
 #endif
      return NULL;
   }
+  conn->hits = 1;
 
   ret = ldap_initialize(&conn->ldap, pool->ldap_uri);
   if (!conn->ldap){
@@ -270,6 +272,7 @@ LDAP *ldap_connection_open(struct ldap_connections_pool *pool)
 #endif
      ci_debug_printf(1, "Error allocating memory for ldap connection: %s!\n",
 		     ldap_err2string(ret));
+     free(conn);
      return NULL;
   }
 
@@ -376,6 +379,27 @@ struct ldap_table_data {
     ci_cache_t *cache;
 };
 
+void str_trim(char *str)
+{
+    char *s, *e;
+
+    if (!str)
+        return;
+
+    s = str;
+    e = NULL;
+    while (*s == ' ' && s != '\0'){
+        e = s;
+        while (*e != '\0'){
+            *e = *(e+1);
+            e++;
+        }
+    }
+
+    /*if (e) e--;  else */
+    e = str+strlen(str);
+    while(*(--e) == ' ' && e >= str) *e = '\0';
+}
 
 int parse_ldap_str(struct ldap_table_data *fields)
 {
@@ -386,18 +410,6 @@ int parse_ldap_str(struct ldap_table_data *fields)
 
     if(!fields->str)
 	return 0;
-
-    s = fields->str;		/*Remove the ' '*/
-    while (*s != '\0'){
-	while (*s == ' '){
-	    e = s;
-	    while (*e != '\0'){
-		*e = *(e+1);
-		e++;
-	    }
-	}
-	s++;
-    }
     i = 0;
     s = fields->str;
     while(*s == '/'){    /*Ignore "//" at the beginning*/
@@ -415,22 +427,28 @@ int parse_ldap_str(struct ldap_table_data *fields)
 	if ((e=strchr(fields->user, ':')) != NULL) {
 	  *e = '\0';
 	  fields->password = e + 1;
+          str_trim(fields->password);
 	}
+        str_trim(fields->user); /* here we have parsed the user*/
     }
-
+    
     fields->server = s;	 /*The s points to the "server" field now*/
     while(*s != '?' && *s != '/'  && *s != '\0') s++;
     if (*s == '\0') 
 	return 0;
     *s = '\0';
+    str_trim(fields->server);
+
     s++;
     fields->base = s;	 /*The s points to the "base" field now*/
     while(*s != '?' && *s != '\0') s++;
     if (*s == '\0') 
 	return 0;
     *s = '\0';
+    str_trim(fields->base);
+
     s++;
-    e = s;			    /*Count the args*/
+    e = s;	/*Count the args*/
     array_size = 1;	
     while (*e != '?' && *e != '\0'){
 	if (*e == ',') 
@@ -452,11 +470,16 @@ int parse_ldap_str(struct ldap_table_data *fields)
 	fields->attrs[i] = s;  /*Every pointer of the array points to an "arg", the last points NULL*/
 	i++;
     }
-    fields->attrs[i] = NULL;
-    while (*s != '?') s++;												
+    while (*s != '?') s++;											
     *s = '\0';
+
+    fields->attrs[i] = NULL;
+    for(i=0; fields->attrs[i] != NULL; i++)
+        str_trim(fields->attrs[i]);
+
     s++;
-    fields->filter = s;		/*The s points to the "filter" field now*/
+    fields->filter = s;   /*The s points to the "filter" field now*/
+    str_trim(fields->filter);
     return 1;
 }
 
@@ -477,6 +500,10 @@ void *store_val(void *val,int *val_size, ci_mem_allocator_t *allocator)
     *val_size = indx_size + cache_val->size;
 
     data = allocator->alloc(allocator, *val_size);
+    if (!data) {
+        ci_debug_printf(1, "Memory allocation failed inside ldap_module.c:store_val() \n");
+        return NULL;
+    }
     data_start = data + indx_size;
     data_indx = data;
     val_data = cache_val->data;
@@ -493,7 +520,7 @@ void *store_val(void *val,int *val_size, ci_mem_allocator_t *allocator)
 
 void *read_val(void *val, int val_size, ci_mem_allocator_t *allocator)
 {
-    char *data = (char *)malloc(MAX_DATA_SIZE);
+    char *data = (char *)ci_buffer_alloc(MAX_DATA_SIZE);;
     char **indx;
     memcpy(data, val, val_size);
     indx=(char **)data;
@@ -503,7 +530,7 @@ void *read_val(void *val, int val_size, ci_mem_allocator_t *allocator)
 #if SIZEOF_VOID_P == 8
 	*indx = (char *) ((uint64_t)*indx+(uint64_t)data);
 #else
-	*indx = (char *) ((uint32_t)*indx+(uint32_t)data);
+	*indx = (char *) ((unsigned int)*indx+(unsigned int)data);
 #endif
 	indx++;
     }
@@ -515,6 +542,7 @@ void *ldap_table_open(struct ci_lookup_table *table)
 {
     char *path;
     struct ldap_table_data *ldapdata;
+    int use_cache = 1;
     
     path = strdup(table->path);
     if (!path) {
@@ -522,6 +550,12 @@ void *ldap_table_open(struct ci_lookup_table *table)
        return NULL;
     }
 
+    if (table->args) {
+        if(strstr(table->args, "cache=no")) {
+            use_cache = 0;
+        }
+    }
+
     ldapdata = malloc(sizeof(struct ldap_table_data));
     if(!ldapdata) {
 	free(path);
@@ -606,7 +640,6 @@ int create_filter(char *filter,int size, char *frmt,char *key)
 
 void *ldap_table_search(struct ci_lookup_table *table, void *key, void ***vals)
 {
-    //struct ci_mem_allocator *allocator = table->allocator;
     struct ldap_table_data *data = (struct ldap_table_data *)table->data;
     LDAPMessage *msg, *entry;
     BerElement *aber;
@@ -625,7 +658,7 @@ void *ldap_table_search(struct ci_lookup_table *table, void *key, void ***vals)
     memory_exhausted = 0;
     return_value = NULL;
 
-    if(data->cache && ci_cache_search(data->cache, key, (void *)vals, NULL)) {
+    if(data->cache && ci_cache_search(data->cache, key, (void **)vals, NULL)) {
 	ci_debug_printf(4, "Retrieving from cache....\n");
 	return key;
     }
@@ -649,7 +682,7 @@ void *ldap_table_search(struct ci_lookup_table *table, void *key, void ***vals)
 
 	ci_debug_printf(4, "Contacting LDAP server: %s\n", ldap_err2string(ret));
 	if(ret == LDAP_SUCCESS) {
-	    indx_values = (char *)malloc(MAX_DATA_SIZE);
+	    indx_values = (char *)ci_buffer_alloc(MAX_DATA_SIZE);
 	    data_values = indx_values + DATA_START;
 	    packer = ci_create_pack_allocator(data_values,DATA_SIZE);
 	    *vals=(void * *)indx_values;
@@ -724,7 +757,5 @@ void *ldap_table_search(struct ci_lookup_table *table, void *key, void ***vals)
 
 void  ldap_table_release_result(struct ci_lookup_table *table,void **val)
 {
-//    struct ci_mem_allocator *allocator = table->allocator;
-//    allocator->free(allocator, val);
-    free(val);
+    ci_buffer_free(val);
 }
