@@ -41,14 +41,13 @@
 #include "cfg_param.h"
 #include "commands.h"
 
+#define MULTICHILD
+//#undef MULTICHILD
+
+
 extern int KEEPALIVE_TIMEOUT;
 extern int MAX_KEEPALIVE_REQUESTS;
 extern int MAX_SECS_TO_LINGER;
-extern int START_CHILDS;
-extern int MAX_CHILDS;
-extern int START_SERVERS;
-extern int MIN_FREE_SERVERS;
-extern int MAX_FREE_SERVERS;
 extern int MAX_REQUESTS_BEFORE_REALLOCATE_MEM;
 extern int MAX_REQUESTS_PER_CHILD;
 extern struct ci_server_conf CONF;
@@ -72,10 +71,11 @@ int listener_running = 0;
 ci_thread_cond_t free_server_cond;
 ci_thread_mutex_t counters_mtx;
 
-struct childs_queue *childs_queue;
+struct childs_queue *childs_queue = NULL;
 struct childs_queue *old_childs_queue = NULL;
-child_shared_data_t *child_data;
+child_shared_data_t *child_data = NULL;
 struct connections_queue *con_queue;
+process_pid_t MY_PROC_PID = 0;
 /*Child shutdown timeout is 10 seconds:*/
 const int CHILD_SHUTDOWN_TIMEOUT = 10;
 int CHILD_HALT = 0;
@@ -100,7 +100,9 @@ void system_shutdown();
 
 static void term_handler_child(int sig)
 {
-     ci_debug_printf(5, "A termination signal received (%d).\n", sig);
+     if (!child_data)
+         return; /*going down?*/
+
      if (!child_data->father_said)
           child_data->to_be_killed = IMMEDIATELY;
      else
@@ -109,24 +111,19 @@ static void term_handler_child(int sig)
 
 static void sigpipe_handler(int sig)
 {
-     ci_debug_printf(5, "SIGPIPE signal received.\n");
 }
 
 static void empty(int sig)
 {
-     ci_debug_printf(10, "An empty signal handler (%d).\n", sig);
 }
 
 static void sigint_handler_main(int sig)
 {
      if (sig == SIGTERM) {
-          ci_debug_printf(5, "SIGTERM signal received for main server.\n");
      }
      else if (sig == SIGINT) {
-          ci_debug_printf(5, "SIGINT signal received for main server.\n");
      }
      else {
-          ci_debug_printf(5, "Signal %d received. Exiting ....\n", sig);
      }
 
      c_icap_going_to_term = 1;
@@ -184,8 +181,11 @@ void thread_signals(int islistener)
 
 static void exit_normaly()
 {
-     dettach_childs_queue(childs_queue);
      system_shutdown();
+#ifdef MULTICHILD
+     child_data = NULL;
+     dettach_childs_queue(childs_queue);
+#endif
 }
 
 static void release_thread_i(int i)
@@ -234,13 +234,13 @@ static void cancel_all_threads()
      ci_thread_cond_broadcast(&(con_queue->queue_cond));
      /*wait for a milisecond*/
      ci_usleep(1000);
-     servers_running = START_SERVERS;
+     servers_running = CONF.THREADS_PER_CHILD;
      while (servers_running && wait_for_workers >= 0) {
          /*child_data->to_be_killed, may change while we are inside this loop*/
          if (child_data->to_be_killed == IMMEDIATELY) {
              CHILD_HALT = 1;
          }
-         for (i=0; i<START_SERVERS; i++) {
+         for (i=0; i<CONF.THREADS_PER_CHILD; i++) {
              if (threads_list[i] != NULL) { /* if the i thread is still alive*/
                  if (!threads_list[i]->running) { /*if the i thread is not running any more*/
                      ci_debug_printf(5, "Cancel server %d, thread_id %lu (%d)\n",
@@ -250,7 +250,7 @@ static void cancel_all_threads()
                      release_thread_i(i);
                      servers_running --;
                  }
-                 else if (child_data->to_be_killed == IMMEDIATELY){
+                 else if (child_data->to_be_killed == IMMEDIATELY){ 
                      /*The thread is still running, and we have a timeout for waiting 
                        the thread to exit. */
                      if (wait_for_workers <= 2) {
@@ -259,7 +259,7 @@ static void cancel_all_threads()
                      }
                  }
              }/*the i thread is still alive*/
-         } /* for(i=0;i< START_SERVERS;i++)*/
+         } /* for(i=0;i< CONF.THREADS_PER_CHILD;i++)*/
 
          /*wait for 1 second for the next round*/
          ci_usleep(999999);
@@ -282,46 +282,93 @@ static void cancel_all_threads()
      }
 }
 
+static void send_term_to_childs(struct childs_queue *q)
+{
+    int i, pid;
+    for (i = 0; i < q->size; i++) {
+        if ((pid = q->childs[i].pid) == 0)
+            continue;
+        if (q->childs[i].to_be_killed != IMMEDIATELY) {
+            /*Child did not informed yet*/
+            q->childs[i].father_said = IMMEDIATELY;
+            kill(pid, SIGTERM);
+        }
+    }
+}
+
+static void wait_childs_to_exit(struct childs_queue *q)
+{
+    int i, status, pid;
+    for (i = 0; i < q->size; i++) {
+        if (q->childs[i].pid == 0)
+            continue;
+        ci_debug_printf(5, "Wait for child with pid:%d\n", q->childs[i].pid);
+        if (q->childs[i].to_be_killed != IMMEDIATELY) {
+            ci_debug_printf(5, "Child %d not signaled yet!\n", q->childs[i].pid);
+            continue;
+        }
+
+        do {
+            errno = 0;
+            pid = waitpid(q->childs[i].pid, &status, WNOHANG);
+        } while (pid < 0 && errno == EINTR);
+
+        if (pid > 0) {
+            remove_child(q, pid, 0);
+            ci_debug_printf(5, "Child %d died with status %d\n", pid, status);
+        }
+    }
+}
+
 static void kill_all_childs()
 {
-     int i = 0, status, pid;
+    int childs_running;
      ci_debug_printf(5, "Going to term children....\n");
-     for (i = 0; i < childs_queue->size; i++) {
-          if ((pid = childs_queue->childs[i].pid) == 0)
-               continue;
-          /*listener thread will be interupted it is good to know 
-             that it will going down imediatelly */
-          childs_queue->childs[i].father_said = IMMEDIATELY;
-          kill(pid, SIGTERM);
-     }
 
-     for (i = 0; i < childs_queue->size; i++) {
-          if (childs_queue->childs[i].pid == 0)
-               continue;
-          pid = wait(&status);
-          ci_debug_printf(5, "Child %d died with status %d\n", pid, status);
-     }
+     childs_running = 0;
+     do {
+         send_term_to_childs(childs_queue);
+         if (old_childs_queue)
+             send_term_to_childs(old_childs_queue);
+
+         /*wait for 30 milisecond for childs to take care*/
+         ci_usleep(30000);
+
+         wait_childs_to_exit(childs_queue);
+         childs_running = !childs_queue_is_empty(childs_queue);
+         if (old_childs_queue) {
+             wait_childs_to_exit(old_childs_queue);
+             childs_running += !childs_queue_is_empty(old_childs_queue);
+         }
+     } while(childs_running);
+
      ci_proc_mutex_destroy(&accept_mutex);
      destroy_childs_queue(childs_queue);
+     childs_queue = NULL;
+     if (old_childs_queue)
+         destroy_childs_queue(old_childs_queue);
+     old_childs_queue = NULL;
 }
 
 static void check_for_exited_childs()
 {
-     int status, pid, ret;
+    int status, pid, ret, exit_status;
+    exit_status = 0;
      while ((pid = waitpid(-1, &status, WNOHANG)) > 0) {
           ci_debug_printf(5, "Child %d died ...\n", pid);
           if (!WIFEXITED(status)) {
                ci_debug_printf(1, "Child %d did not exit normally.", pid);
+               exit_status = 1;
                if (WIFSIGNALED(status))
                     ci_debug_printf(1, "signaled with signal:%d\n",
                                     WTERMSIG(status));
           }
-          ret = remove_child(childs_queue, pid);
+          ret = remove_child(childs_queue, pid, exit_status);
           if (ret == 0 && old_childs_queue) {
                ci_debug_printf(5,
                                "Child %d will be removed from the old list ...\n",
                                pid);
-               remove_child(old_childs_queue, pid);
+               remove_child(old_childs_queue, pid, exit_status);
                if (childs_queue_is_empty(old_childs_queue)) {
                     ret = destroy_childs_queue(old_childs_queue);
                     /* if(!ret){} */
@@ -366,7 +413,7 @@ static int server_reconfigure()
       */
      old_childs_queue = childs_queue;
      childs_queue = malloc(sizeof(struct childs_queue));
-     if (!create_childs_queue(childs_queue, 2 * MAX_CHILDS)) {
+     if (!create_childs_queue(childs_queue, 2 * CONF.MAX_SERVERS)) {
           ci_debug_printf(1,
                           "Cannot init shared memory. Fatal error, exiting!\n");
           return 0;              /*It is not enough. We must wait all childs to exit ..... */
@@ -374,10 +421,10 @@ static int server_reconfigure()
      /*
         Start new childs to handle new requests.
       */
-     if (START_CHILDS > MAX_CHILDS)
-          START_CHILDS = MAX_CHILDS;
+     if (CONF.START_SERVERS > CONF.MAX_SERVERS)
+          CONF.START_SERVERS = CONF.MAX_SERVERS;
 
-     for (i = 0; i < START_CHILDS; i++) {
+     for (i = 0; i < CONF.START_SERVERS; i++) {
           start_child(LISTEN_SOCKET);
      }
 
@@ -398,7 +445,7 @@ int ci_named_pipe_create(char *name)
 {
      int status, pipe;
      errno = 0;
-     status = mkfifo(name, S_IRWXU | S_IRGRP | S_IROTH);
+     status = mkfifo(name, S_IRUSR | S_IWUSR | S_IWGRP);
      if (status < 0 && errno != EEXIST)
           return -1;
      pipe = open(name, O_RDONLY | O_NONBLOCK);
@@ -417,7 +464,7 @@ void ci_named_pipe_close(int pipe_fd)
      close(pipe_fd);
 }
 
-int wait_for_commands(int pipe_fd, char *command_buffer, int secs)
+int wait_for_commands(int ctl_fd, char *command_buffer, int secs)
 {
      fd_set fds;
      struct timeval tv;
@@ -428,29 +475,34 @@ int wait_for_commands(int pipe_fd, char *command_buffer, int secs)
           tv.tv_usec = 0;
      }
      FD_ZERO(&fds);
-     FD_SET(pipe_fd, &fds);
+     FD_SET(ctl_fd, &fds);
      errno = 0;
      if ((ret =
-          select(pipe_fd + 1, &fds, NULL, NULL,
+          select(ctl_fd + 1, &fds, NULL, NULL,
                  (secs >= 0 ? &tv : NULL))) > 0) {
-          if (FD_ISSET(pipe_fd, &fds)) {
+          if (FD_ISSET(ctl_fd, &fds)) {
                ret =
-                   ci_read_nonblock(pipe_fd, command_buffer,
+                   ci_read_nonblock(ctl_fd, command_buffer,
                                     COMMANDS_BUFFER_SIZE - 1);
-               if (ret >=0) {
+               if (ret > 0) {
                    command_buffer[ret] = '\0';
                    return ret;
                }
+
+               if (ret == 0 ) {
+                /*read return 0, This is an eof, we must return -1 to force socket reopening!*/
+                   return -1;
+               }
           }
      }
      if (ret < 0 && errno != EINTR) {
           ci_debug_printf(1,
-                          "Unxpected error waiting for events in control socket!\n");
-          /*returning 0 we are causing reopening control socket! */
-          return 0;
+                          "Unexpected error waiting for or reading  events in control socket!\n");
+          /*returning -1 we are causing reopening control socket! */
+          return -1;
      }
 
-     return -1;                 /*expired */
+     return 0; /*expired */
 }
 
 void handle_monitor_process_commands(char *cmd_line)
@@ -482,7 +534,7 @@ void handle_child_process_commands(char *cmd_line)
 
 
 /*************************************************************************************/
-/*Childs  functions                                                                  */
+/*Children  functions                                                                  */
 
 server_decl_t *newthread(struct connections_queue *con_queue)
 {
@@ -502,7 +554,7 @@ int thread_main(server_decl_t * srv)
 {
      ci_connection_t con;
      char clientname[CI_MAXHOSTNAMELEN + 1];
-     int ret, request_status = 0;
+     int ret, request_status = CI_NO_STATUS;
      int keepalive_reqs;
 //***********************
      thread_signals(0);
@@ -563,11 +615,11 @@ int thread_main(server_decl_t * srv)
                if (child_data->to_be_killed)    /*We are going to die do not keep-alive */
                     srv->current_req->keepalive = 0;
 
-               if ((request_status = process_request(srv->current_req)) < 0) {
+               if ((request_status = process_request(srv->current_req)) == CI_NO_STATUS) {
                     ci_debug_printf(5,
                                     "Process request timeout or interrupted....\n");
                     ci_request_reset(srv->current_req);
-                    break;      //
+                    break;
                }
                srv->served_requests++;
                srv->served_requests_no_reallocation++;
@@ -606,7 +658,7 @@ int thread_main(server_decl_t * srv)
           } while (1);
 
           if (srv->current_req) {
-               if (request_status < 0 || child_data->to_be_killed) {
+               if (request_status != CI_OK || child_data->to_be_killed) {
                     hard_close_connection(srv->current_req->connection);
                }
                else {
@@ -643,6 +695,11 @@ void listener_thread(int *fd)
      int pid, sockfd;
      sockfd = *fd;
      thread_signals(1);
+     /*Wait main process to signal us to start accepting requests*/
+     ci_thread_mutex_lock(&counters_mtx);
+     listener_running = 1;
+     ci_thread_cond_wait(&free_server_cond, &counters_mtx);
+     ci_thread_mutex_unlock(&counters_mtx);
      pid = getpid();
      for (;;) {                 //Global for
           if (child_data->to_be_killed) {
@@ -799,13 +856,11 @@ void child_main(int sockfd, int pipefd)
      ci_stat_attach_mem(child_data->stats, child_data->stats_size, NULL);
 
      threads_list =
-         (server_decl_t **) malloc((START_SERVERS + 1) *
+         (server_decl_t **) malloc((CONF.THREADS_PER_CHILD + 1) *
                                    sizeof(server_decl_t *));
-     con_queue = init_queue(START_SERVERS);
+     con_queue = init_queue(CONF.THREADS_PER_CHILD);
 
-     execure_start_child_commands ();
-
-     for (i = 0; i < START_SERVERS; i++) {
+     for (i = 0; i < CONF.THREADS_PER_CHILD; i++) {
           if ((threads_list[i] = newthread(con_queue)) == NULL) {
                exit(-1);        // FATAL error.....
           }
@@ -814,31 +869,52 @@ void child_main(int sockfd, int pipefd)
                                (void *) threads_list[i]);
           threads_list[i]->srv_pthread = thread;
      }
-     threads_list[START_SERVERS] = NULL;
+     threads_list[CONF.THREADS_PER_CHILD] = NULL;
      /*Now start the listener thread.... */
      ret = ci_thread_create(&thread, (void *(*)(void *)) listener_thread,
                             (void *) &sockfd);
      listener_thread_id = thread;
-     listener_running = 1;
      
      /*set srand for child......*/
      srand(((unsigned int)time(NULL)) + (unsigned int)getpid());
      /*I suppose that all my threads are up now. We can setup our signal handlers */
      child_signals();
-     for (;;) {
+
+     /* A signal from parent may comes while we are starting.
+        Listener will not accept any request in this case, (it checks on 
+        the beggining of the accept loop for parent commands) so we can 
+        shutdown imediatelly even if the parent said gracefuly.*/
+     if (child_data->father_said)
+         child_data->to_be_killed = IMMEDIATELY;
+
+     /*start child commands may have non thread safe code but the worker threads
+       does not serving requests yet.*/
+     commands_execute_start_child();
+
+     /*Signal listener to start accepting requests.*/
+     int doStart = 0;
+     do {
+         ci_thread_mutex_lock(&counters_mtx);
+         doStart = listener_running;
+         ci_thread_mutex_unlock(&counters_mtx);
+         if (!doStart)
+             ci_usleep(5);
+     } while(!doStart);
+     ci_thread_cond_signal(&free_server_cond);
+
+     while (!child_data->to_be_killed) {
           char buf[512];
           int bytes;
-          if ((ret = ci_wait_for_data(pipefd, 5, wait_for_read)) > 0) { /*data input */
+          if ((ret = ci_wait_for_data(pipefd, 1, wait_for_read)) > 0) { /*data input */
                bytes = ci_read_nonblock(pipefd, buf, 511);
                if (bytes == 0) {
                     ci_debug_printf(1,
                                     "Parent closed the pipe connection! Going to term immediately!\n");
                     child_data->to_be_killed = IMMEDIATELY;
-                    break;
+               } else {
+                    buf[bytes] = '\0';
+                    handle_child_process_commands(buf);
                }
-               buf[bytes] = '\0';
-               handle_child_process_commands(buf);
-//               ci_debug_printf(1, "Coming data: %s\n", buf);
           }
           else if (ret < 0) {
                ci_debug_printf(1,
@@ -850,17 +926,7 @@ void child_main(int sockfd, int pipefd)
                                "Ohh!! something happened to listener thread! Terminating\n");
                child_data->to_be_killed = GRACEFULLY;
           }
-/*
-          if (MAX_REQUESTS_PER_CHILD > 0
-              && child_data->requests > MAX_REQUESTS_PER_CHILD) {
-               ci_debug_printf(5,
-                               "Maximum number of requests reached.Going down\n");
-               child_data->to_be_killed = GRACEFULLY;
-          }
-*/
-          if (child_data->to_be_killed)
-               break;
-//          ci_debug_printf(5, "Do some tests\n");
+          commands_exec_scheduled();
      }
 
      ci_debug_printf(5, "Child :%d going down :%s\n", getpid(),
@@ -868,6 +934,7 @@ void child_main(int sockfd, int pipefd)
                      "IMMEDIATELY" : "GRACEFULLY");
      
      cancel_all_threads();
+     commands_execute_stop_child();
      exit_normaly();
 }
 
@@ -875,9 +942,6 @@ void child_main(int sockfd, int pipefd)
 /*****************************************************************************************/
 /*Main process functions                                                                 */
 
-#define MULTICHILD
-//#undef MULTICHILD
-
 int start_child(int fd)
 {
      int pid;
@@ -895,38 +959,40 @@ int start_child(int fd)
           close(pfd[1]);
      }
      if ((pid = fork()) == 0) { //A Child .......
+          MY_PROC_PID = getpid();
           attach_childs_queue(childs_queue);
           child_data =
-              register_child(childs_queue, getpid(), START_SERVERS, pfd[1]);
+              register_child(childs_queue, getpid(), CONF.THREADS_PER_CHILD, pfd[1]);
           close(pfd[1]);
           child_main(fd, pfd[0]);
           exit(0);
      }
      else {
           close(pfd[0]);
+          announce_child(childs_queue, pid);
           return pid;
      }
 }
 
-void stop_command(char *name, int type, char **argv)
+void stop_command(const char *name, int type, const char **argv)
 {
      c_icap_going_to_term = 1;
 }
 
-void reconfigure_command(char *name, int type, char **argv)
+void reconfigure_command(const char *name, int type, const char **argv)
 {
      if (type == MONITOR_PROC_CMD)
 	  c_icap_reconfigure = 1;
 	 //server_reconfigure();
 }
 
-void dump_statistics_command(char *name, int type, char **argv)
+void dump_statistics_command(const char *name, int type, const char **argv)
 {
      if (type == MONITOR_PROC_CMD)
           dump_queue_statistics(childs_queue);
 }
 
-void test_command(char *name, int type, char **argv)
+void test_command(const char *name, int type, const char **argv)
 {
      int i = 0;
      ci_debug_printf(1, "Test command for %s. Arguments:",
@@ -973,9 +1039,14 @@ int start_server()
           exit(0);
      }
 
-     ci_proc_mutex_init(&accept_mutex);
+     if (!ci_proc_mutex_init(&accept_mutex)) {
+          ci_debug_printf(1,
+                          "Can't init mutex for accepting conenctions. Fatal error, exiting!\n");
+          exit(0);
+     }
      childs_queue = malloc(sizeof(struct childs_queue));
-     if (!create_childs_queue(childs_queue, 2 * MAX_CHILDS)) {
+     if (!create_childs_queue(childs_queue, 2 * CONF.MAX_SERVERS)) {
+          ci_proc_mutex_destroy(&accept_mutex);
           ci_debug_printf(1,
                           "Can't init shared memory. Fatal error, exiting!\n");
           exit(0);
@@ -984,10 +1055,10 @@ int start_server()
      init_commands();
      pid = 1;
 #ifdef MULTICHILD
-     if (START_CHILDS > MAX_CHILDS)
-          START_CHILDS = MAX_CHILDS;
+     if (CONF.START_SERVERS > CONF.MAX_SERVERS)
+          CONF.START_SERVERS = CONF.MAX_SERVERS;
 
-     for (i = 0; i < START_CHILDS; i++) {
+     for (i = 0; i < CONF.START_SERVERS; i++) {
           if (pid)
                pid = start_child(LISTEN_SOCKET);
      }
@@ -1000,7 +1071,7 @@ int start_server()
                                     command_buffer);
                     handle_monitor_process_commands(command_buffer);
                }
-               if (ret == 0) {  /*Eof received on pipe. Going to reopen ... */
+               if (ret < 0) {  /*Eof received on pipe. Going to reopen ... */
                     ci_named_pipe_close(ctl_socket);
                     ctl_socket = ci_named_pipe_open(CONF.COMMANDS_SOCKET);
                     if (ctl_socket < 0) {
@@ -1034,16 +1105,16 @@ int start_server()
                     kill(childs_queue->childs[child_indx].pid, SIGTERM);
 
                }
-               else if ((freeservers <= MIN_FREE_SERVERS && childs < MAX_CHILDS)
-                        || childs < START_CHILDS) {
+               else if ((freeservers <= CONF.MIN_SPARE_THREADS && childs < CONF.MAX_SERVERS)
+                        || childs < CONF.START_SERVERS) {
                     ci_debug_printf(8,
                                     "Free Servers: %d, children: %d. Going to start a child .....\n",
                                     freeservers, childs);
                     pid = start_child(LISTEN_SOCKET);
                }
-               else if (freeservers >= MAX_FREE_SERVERS &&
-                        childs > START_CHILDS &&
-                        (freeservers - START_SERVERS) > MIN_FREE_SERVERS) {
+               else if (freeservers >= CONF.MAX_SPARE_THREADS &&
+                        childs > CONF.START_SERVERS &&
+                        (freeservers - CONF.THREADS_PER_CHILD) > CONF.MIN_SPARE_THREADS) {
 
                     if ((child_indx = find_an_idle_child(childs_queue)) >= 0) {
                          childs_queue->childs[child_indx].father_said =
@@ -1058,7 +1129,7 @@ int start_server()
 			 user_informed = 0;
                     }
                }
-               else if (childs == MAX_CHILDS && freeservers < MIN_FREE_SERVERS) {
+               else if (childs == CONF.MAX_SERVERS && freeservers < CONF.MIN_SPARE_THREADS) {
 		 if(! user_informed) {
 		         ci_debug_printf(1,
 					 "ATTENTION!!!! Not enough available servers (children %d, free servers %d, used servers %d)!!!!! "
@@ -1089,14 +1160,20 @@ int start_server()
 #else
      child_data = (child_shared_data_t *) malloc(sizeof(child_shared_data_t));
      child_data->pid = 0;
-     child_data->freeservers = START_SERVERS;
+     child_data->freeservers = CONF.THREADS_PER_CHILD;
      child_data->usedservers = 0;
      child_data->requests = 0;
      child_data->connections = 0;
      child_data->to_be_killed = 0;
      child_data->father_said = 0;
      child_data->idle = 1;
+     child_data->stats_size = ci_stat_memblock_size();
+     child_data->stats = malloc(child_data->stats_size);
+     child_data->stats->sig = MEMBLOCK_SIG;
+     ci_stat_attach_mem(child_data->stats, child_data->stats_size, NULL);
      child_main(LISTEN_SOCKET, 0);
+     ci_proc_mutex_destroy(&accept_mutex);
+     destroy_childs_queue(childs_queue);
 #endif
      return 1;
 }
