@@ -27,6 +27,20 @@
 #include "request.h"
 #include "simple_api.h"
 
+static void * _os_malloc(int size)
+{
+     return malloc(size);
+}
+
+static void _os_free(void *ptr)
+{
+     free(ptr);
+}
+
+void *(*__intl_malloc)(int) = _os_malloc;
+void (*__intl_free)(void *)  = _os_free;
+
+
 /* struct buf functions*/
 void ci_buf_init(struct ci_buf *buf)
 {
@@ -39,10 +53,9 @@ void ci_buf_reset(struct ci_buf *buf)
 {
      buf->used = 0;
 }
-
 int ci_buf_mem_alloc(struct ci_buf *buf, int size)
 {
-     if (!(buf->buf = malloc(size * sizeof(char))))
+     if (!(buf->buf = __intl_malloc(size * sizeof(char))))
           return 0;
      buf->size = size;
      buf->used = 0;
@@ -51,7 +64,7 @@ int ci_buf_mem_alloc(struct ci_buf *buf, int size)
 
 void ci_buf_mem_free(struct ci_buf *buf)
 {
-     free(buf->buf);
+     __intl_free(buf->buf);
      buf->buf = NULL;
      buf->size = 0;
      buf->used = 0;
@@ -72,7 +85,7 @@ int ci_buf_reset_size(struct ci_buf *buf, int req_size)
      if (buf->size > req_size)
           return req_size;
      if (buf->buf)
-          free(buf->buf);
+          __intl_free(buf->buf);
      return ci_buf_mem_alloc(buf, req_size);
 }
 
@@ -175,6 +188,11 @@ ci_encaps_entity_t *ci_request_alloc_entity(ci_request_t * req, int type, int va
           req->trash_entities[type] = NULL;
           e->type = type;
           e->start = val;
+          if (e->type == ICAP_REQ_HDR
+              || e->type == ICAP_RES_HDR) {
+              if (e->entity)
+                  ci_headers_reset((ci_headers_list_t *)e->entity);
+          }
           ci_debug_printf(8, "Get entity from trash....\n");
           return e;
      }
@@ -205,11 +223,6 @@ int ci_request_release_entity(ci_request_t * req, int pos)
           destroy_encaps_entity(req->trash_entities[type]);
      }
      req->trash_entities[type] = req->entities[pos];
-     if (req->trash_entities[type]->type == ICAP_REQ_HDR
-         || req->trash_entities[type]->type == ICAP_RES_HDR) {
-          if (req->trash_entities[type]->entity)
-               ci_headers_reset(req->trash_entities[type]->entity);
-     }
      req->entities[pos] = NULL;
      return 1;
 }
@@ -219,7 +232,9 @@ ci_request_t *ci_request_alloc(ci_connection_t * connection)
 {
      ci_request_t *req;
      int i;
-     req = (ci_request_t *) malloc(sizeof(ci_request_t));
+     req = (ci_request_t *) __intl_malloc(sizeof(ci_request_t));
+     if (!req)
+         return NULL;
     
      req->connection = connection;
      req->packed = 0;
@@ -228,6 +243,7 @@ ci_request_t *ci_request_alloc(ci_connection_t * connection)
      req->access_type = 0;
 
      req->service[0] = '\0';
+     req->req_server[0] = '\0';
      req->current_service_mod = NULL;
      req->service_data = NULL;
      req->args[0] = '\0';
@@ -237,6 +253,7 @@ ci_request_t *ci_request_alloc(ci_connection_t * connection)
 
      req->keepalive = 1;        /*Keep alive connection is the default behaviour for icap protocol. */
      req->allow204 = 0;
+     req->allow206 = 0;
      req->hasbody = 0;
      req->responce_hasbody = 0;
      req->eof_received = 0;
@@ -256,10 +273,15 @@ ci_request_t *ci_request_alloc(ci_connection_t * connection)
      req->pstrblock_responce = NULL;
      req->remain_send_block_bytes = 0;
      req->data_locked = 1;
+     req->i206_use_original_body = -1;
 
      req->preview_data_type = -1;
      req->auth_required = 0;
 
+     
+     req->log_str = NULL;
+     req->attributes = NULL;
+
      req->bytes_in = 0;
      req->bytes_out = 0;
      req->http_bytes_in = 0;
@@ -296,6 +318,7 @@ void ci_request_reset(ci_request_t * req)
 
      req->keepalive = 1;        /*Keep alive connection is the default behaviour for icap protocol. */
      req->allow204 = 0;
+     req->allow206 = 0;
      req->hasbody = 0;
      req->responce_hasbody = 0;
      ci_headers_reset(req->request_header);
@@ -313,10 +336,19 @@ void ci_request_reset(ci_request_t * req)
      req->remain_send_block_bytes = 0;
      req->write_to_module_pending = 0;
      req->data_locked = 1;
+     req->i206_use_original_body = -1;
 
      req->preview_data_type = -1;
      req->auth_required = 0;
 
+     if (req->log_str)
+         __intl_free(req->log_str);
+     req->log_str = NULL;
+
+     if (req->attributes)
+         ci_array_destroy(req->attributes);
+     req->attributes = NULL;
+
      req->bytes_in = 0;
      req->bytes_out = 0;
      req->http_bytes_in = 0;
@@ -328,13 +360,20 @@ void ci_request_reset(ci_request_t * req)
           ci_request_release_entity(req, i);
      }
 
+     /*Reset the encapsulated response or request headers*/
+     if (req->trash_entities[ICAP_REQ_HDR] && 
+         req->trash_entities[ICAP_REQ_HDR]->entity)
+         ci_headers_reset((ci_headers_list_t *)req->trash_entities[ICAP_REQ_HDR]->entity);
+     if (req->trash_entities[ICAP_RES_HDR] && 
+         req->trash_entities[ICAP_RES_HDR]->entity)
+         ci_headers_reset((ci_headers_list_t *)req->trash_entities[ICAP_RES_HDR]->entity);
 }
 
 void ci_request_destroy(ci_request_t * req)
 {
      int i;
      if (req->connection)
-          free(req->connection);
+          __intl_free(req->connection);
 
      ci_buf_mem_free(&(req->preview_data));
      ci_headers_destroy(req->request_header);
@@ -348,32 +387,93 @@ void ci_request_destroy(ci_request_t * req)
                destroy_encaps_entity(req->trash_entities[i]);
      }
 
-     free(req);
+     if (req->log_str)
+         __intl_free(req->log_str);
+
+     if (req->attributes)
+         ci_array_destroy(req->attributes);
+
+     __intl_free(req);
 }
 
-
-int process_encapsulated(ci_request_t * req, char *buf)
+char *ci_request_set_log_str(ci_request_t *req, char *logstr)
 {
-     char *start;
-     char *pos;
+     int size;
+     if (req->log_str)
+         __intl_free(req->log_str);
+
+     size = strlen(logstr) + 1;
+     req->log_str = __intl_malloc(size*sizeof(char));
+     if (!req->log_str)
+         return NULL;
+     strcpy(req->log_str, logstr);
+     return req->log_str;
+}
+
+int  ci_request_set_str_attribute(ci_request_t *req, const char *name, const char *value)
+{
+    if (req->attributes == NULL) {
+        req->attributes = ci_array_new(4096);
+        if (!req->attributes) {
+            ci_debug_printf(1, "Error allocating request attributes array!\n");
+            return 0;
+        }
+    }
+
+    if (!ci_str_array_add(req->attributes, name, value)) {
+        ci_debug_printf(1, "Not enough space to add attribute %s:%s for service %s\n", name, value, req->service);
+        return 0;
+    }
+    return 1;
+}
+
+int ci_request_206_origin_body(ci_request_t *req, uint64_t offset)
+{
+    if (!req)
+        return 0;
+    
+    if (!req->allow206) {
+        ci_debug_printf(1, "Request does not support allow206 responses! Can not set use-original-body extension\n");
+        return 0;
+    }
+
+#if 0
+    if (req->body_bytes_in < offset) {
+        ci_debug_printf(1, "Can not set use-original-body extension to offset longer than the known body size");
+        return 0;
+    }
+#endif
+    req->i206_use_original_body = offset;
+    return 1;
+}
+
+int process_encapsulated(ci_request_t * req, const char *buf)
+{
+     const char *start;
+     const char *pos;
      char *end;
      int type = 0, num = 0, val = 0;
      int hasbody = 1;           /*Assume that we have a resbody or reqbody or optbody */
      start = buf + 14;
      pos = start;
+     end = (char *)start;
      while (*pos != '\0') {
           while (!isalpha(*pos) && *pos != '\0')
                pos++;
           type = get_encaps_type(pos, &val, &end);
+          if (type < 0) /*parse error - return "400 bad request"*/
+              return EC_400;
           if (num > 5)          /*In practice there is an error here .... */
                break;
           if (type == ICAP_NULL_BODY)
                hasbody = 0;     /*We have not a body */
           req->entities[num++] = ci_request_alloc_entity(req, type, val);
+
+          assert(start != end);
           pos = end;            /* points after the value of entity.... */
      }
      req->hasbody = hasbody;
-     return 0;
+     return EC_100;
 }
 
 
@@ -421,12 +521,14 @@ int parse_chunk_data(ci_request_t * req, char **wdata)
                if (req->current_chunk_len == 0) {
 
                     if (*end == ';') {
-                         if (req->pstrblock_read_len < 11) {    /*must hold a 0; ieof\r\n\r\n */
-                              return CI_NEEDS_MORE;
-                         }
+			 if(strnstr(end, "\r\n\r\n", req->pstrblock_read_len) == NULL)
+			      return CI_NEEDS_MORE;
 
-                         if (strncmp(end, "; ieof", 6) != 0)
-                              return CI_ERROR;
+			 if (strncmp(end, "; use-original-body=", 20) == 0) {
+			     req->i206_use_original_body = strtol(end+20, NULL, 10);
+		         }
+			 else if (strncmp(end, "; ieof", 6) != 0)
+			      return CI_ERROR;
 
                          req->eof_received = 1;
                          return CI_EOF;
@@ -520,7 +622,7 @@ int net_data_read(ci_request_t * req)
      }
 
      if ((bytes = ci_read_nonblock(req->connection->fd, req->rbuf + req->pstrblock_read_len, bytes)) <= 0) {    /*... read some data... */
-          ci_debug_printf(5, "Error reading data (read return=%d) \n", bytes);
+          ci_debug_printf(5, "Error reading data (read return=%d, errno=%d) \n", bytes, errno);
           return CI_ERROR;
      }
      req->pstrblock_read_len += bytes;  /* ... (size of data is readed plus old )... */
@@ -540,6 +642,10 @@ ci_request_t *ci_client_request(ci_connection_t * conn, char *server,
 {
      ci_request_t *req;
      req = ci_request_alloc(conn);
+     if (!req) {
+         ci_debug_printf(1, "Error allocation ci_request_t object(ci_client_request())\n");
+         return NULL;
+     }
      strncpy(req->req_server, server, CI_MAXHOSTNAMELEN);
      req->req_server[CI_MAXHOSTNAMELEN] = '\0';
      strncpy(req->service, service, MAX_SERVICE_NAME);
@@ -574,6 +680,10 @@ void ci_client_request_reuse(ci_request_t * req)
      req->write_to_module_pending = 0;
      req->data_locked = 1;
 
+     req->allow204 = 0;
+     req->allow206 = 0;
+     req->i206_use_original_body = -1;
+
      req->bytes_in = 0;
      req->bytes_out = 0;
      req->http_bytes_in = 0;
@@ -589,7 +699,7 @@ void ci_client_request_reuse(ci_request_t * req)
 
 
 
-int client_create_request(ci_request_t * req, char *servername, char *service,
+static int client_create_request(ci_request_t * req, char *servername, char *service,
                           int reqtype)
 {
      char buf[256];
@@ -607,8 +717,13 @@ int client_create_request(ci_request_t * req, char *servername, char *service,
      buf[255] = '\0';
      ci_headers_add(req->request_header, buf);
      ci_headers_add(req->request_header, "User-Agent: C-ICAP-Client-Library/0.01");
-     if (ci_allow204(req))
+
+     if (ci_allow204(req) && ci_allow206(req))
+          ci_headers_add(req->request_header, "Allow: 204, 206");
+     else if (ci_allow204(req))
           ci_headers_add(req->request_header, "Allow: 204");
+     if (ci_allow206(req))
+          ci_headers_add(req->request_header, "Allow: 206");
 
      if (!ci_headers_is_empty(req->xheaders)) {
 	  ci_headers_addheaders(req->request_header, req->xheaders);
@@ -617,9 +732,9 @@ int client_create_request(ci_request_t * req, char *servername, char *service,
      return CI_OK;
 }
 
-int get_request_options(ci_request_t * req, ci_headers_list_t * h)
+static int get_request_options(ci_request_t * req, ci_headers_list_t * h)
 {
-     char *pstr;
+     const char *pstr;
 
      if ((pstr = ci_headers_value(h, "Preview")) != NULL) {
           req->preview = strtol(pstr, NULL, 10);
@@ -661,7 +776,7 @@ int get_request_options(ci_request_t * req, ci_headers_list_t * h)
 }
 
 
-int ci_writen(int fd, char *buf, int len, int timeout)
+static int ci_writen(int fd, char *buf, int len, int timeout)
 {
      int ret = 0, remains;
      remains = len;
@@ -676,7 +791,7 @@ int ci_writen(int fd, char *buf, int len, int timeout)
 
 
 
-int client_send_request_headers(ci_request_t * req, int has_eof, int timeout)
+static int client_send_request_headers(ci_request_t * req, int has_eof, int timeout)
 {
      ci_encaps_entity_t **elist, *e;
      ci_headers_list_t *headers;
@@ -751,7 +866,7 @@ static int check_realloc(char **buf, int *size, int used, int mustadded)
 }
 
 
-int client_parse_icap_header(ci_request_t * req, ci_headers_list_t * h)
+static int client_parse_icap_header(ci_request_t * req, ci_headers_list_t * h)
 {
      int readed = 0, eoh = 0;
      char *buf, *end;
@@ -781,7 +896,7 @@ int client_parse_icap_header(ci_request_t * req, ci_headers_list_t * h)
      return CI_OK;
 }
 
-int client_parse_encaps_header(ci_request_t * req, ci_headers_list_t * h, int size)
+static int client_parse_encaps_header(ci_request_t * req, ci_headers_list_t * h, int size)
 {
      int remains, readed = 0;
      char *buf_end = NULL;
@@ -847,7 +962,7 @@ int ci_client_get_server_options(ci_request_t * req, int timeout)
 
 ci_connection_t *ci_client_connect_to(char *servername, int port, int proto)
 {
-     ci_connection_t *connection = malloc(sizeof(ci_connection_t));
+     ci_connection_t *connection = __intl_malloc(sizeof(ci_connection_t));
      char hostname[CI_MAXHOSTNAMELEN + 1];
      unsigned int addrlen = 0;
      if (!connection)
@@ -855,13 +970,14 @@ ci_connection_t *ci_client_connect_to(char *servername, int port, int proto)
      connection->fd = socket(proto, SOCK_STREAM, 0);
      if (connection->fd == -1) {
           ci_debug_printf(1, "Error opening socket ....\n");
-          free(connection);
+          __intl_free(connection);
           return NULL;
      }
 
      if (!ci_host_to_sockaddr_t(servername, &(connection->srvaddr), proto)) {
 	  ci_debug_printf(1, "Error getting address info for host %s\n", servername);
-          free(connection);
+          close(connection->fd);
+          __intl_free(connection);
           return NULL;
      }
      ci_sockaddr_set_port(&(connection->srvaddr), port);
@@ -873,7 +989,8 @@ ci_connection_t *ci_client_connect_to(char *servername, int port, int proto)
                                 CI_MAXHOSTNAMELEN);
           ci_debug_printf(1, "Error connecting to socket (host: %s) .....\n",
                           hostname);
-          free(connection);
+          close(connection->fd);
+          __intl_free(connection);
           return NULL;
      }
 
@@ -888,7 +1005,7 @@ ci_connection_t *ci_client_connect_to(char *servername, int port, int proto)
 }
 
 
-int client_prepere_body_chunk(ci_request_t * req, void *data,
+static int client_prepere_body_chunk(ci_request_t * req, void *data,
                               int (*readdata) (void *data, char *, int))
 {
      int chunksize, def_bytes;
@@ -918,11 +1035,12 @@ int client_prepere_body_chunk(ci_request_t * req, void *data,
 }
 
 
-int client_parse_incoming_data(ci_request_t * req, void *data_dest,
+static int client_parse_incoming_data(ci_request_t * req, void *data_dest,
                                int (*dest_write) (void *, char *, int))
 {
      int ret, v1, v2, status, bytes, size;
-     char *buf, *val;
+     char *buf;
+     const char *val;
      ci_headers_list_t *resp_heads;
 
      if (req->status == GET_NOTHING) {
@@ -1033,9 +1151,9 @@ int client_parse_incoming_data(ci_request_t * req, void *data_dest,
      return CI_OK;
 }
 
-const char *eof_str = "0\r\n\r\n";
+static const char *eof_str = "0\r\n\r\n";
 
-int client_send_get_data(ci_request_t * req,
+static int client_send_get_data(ci_request_t * req,
                          int timeout,
                          void *data_source, int (*source_read) (void *, char *,
                                                                 int),
@@ -1116,18 +1234,40 @@ int client_send_get_data(ci_request_t * req,
      return CI_OK;
 }
 
+static int client_build_headers(ci_request_t *req, int has_reqhdr, int has_reshdr, int has_body)
+{
+     int i = 0;
 
+     for (i = 0; i < 4; i++) {
+          if (req->entities[i]) {
+               ci_request_release_entity(req, i);
+          }
+     }
+     i = 0;
+
+     if (has_reqhdr)
+         req->entities[i++] = ci_request_alloc_entity(req, ICAP_REQ_HDR, 0);
+     if (has_reshdr)
+          req->entities[i++] = ci_request_alloc_entity(req, ICAP_RES_HDR, 0);
+     if (has_body)
+          req->entities[i] = ci_request_alloc_entity(req, req->type == ICAP_RESPMOD ? ICAP_RES_BODY : ICAP_REQ_BODY, 0);
+     else
+          req->entities[i] = ci_request_alloc_entity(req, ICAP_NULL_BODY, 0);
+     return 1;
+}
 
 int ci_client_icapfilter(ci_request_t * req,
                          int timeout,
-                         ci_headers_list_t * headers,
+                         ci_headers_list_t * req_headers,
+                         ci_headers_list_t * resp_headers,
                          void *data_source, int (*source_read) (void *, char *,
                                                                 int),
                          void *data_dest, int (*dest_write) (void *, char *,
                                                              int))
 {
      int i, ret, v1, v2, remains, pre_eof = 0, preview_status;
-     char *buf, *val;
+     char *buf;
+     const char *val;
 
      if (CI_OK !=
          client_create_request(req, req->req_server, req->service,
@@ -1157,21 +1297,25 @@ int ci_client_icapfilter(ci_request_t * req,
      if (pre_eof)
           req->eof_received = 1;
 
+     /*Bulid client request structure*/
+     if (!client_build_headers(req, (req_headers != NULL), (resp_headers != NULL), (data_source != NULL)))
+         return CI_ERROR;
+
      /*Add the user supplied headers */
-     if (req->type == ICAP_REQMOD && headers) {
-          ci_http_request_create(req, (data_source!= NULL));
-          for (i = 0; i < headers->used; i++) {
-               ci_http_request_add_header(req, headers->headers[i]);
+     if (req_headers) {
+         ci_debug_printf(5, "Going to add %d request headers\n", req_headers->used);
+          for (i = 0; i < req_headers->used; i++) {
+              ci_debug_printf(8, "Add request header: %s\n", req_headers->headers[i]);
+               ci_http_request_add_header(req, req_headers->headers[i]);
           }
      }
-     else if (headers) {
-          ci_http_response_create(req, 1, 1);
-          for (i = 0; i < headers->used; i++) {
-               ci_http_response_add_header(req, headers->headers[i]);
+     if (resp_headers) {
+         ci_debug_printf(5, "Going to add %d response headers\n", resp_headers->used);
+          for (i = 0; i < resp_headers->used; i++) {
+              ci_debug_printf(8, "Add resp header: %s\n", resp_headers->headers[i]);
+               ci_http_response_add_header(req, resp_headers->headers[i]);
           }
      }
-     else
-          ci_http_response_create(req, 0, 1);
 
      if ((ret = client_send_request_headers(req, pre_eof, timeout)) < 0) {
           return CI_ERROR;
@@ -1196,7 +1340,9 @@ int ci_client_icapfilter(ci_request_t * req,
           sscanf(req->response_header->buf, "ICAP/%d.%d %d", &v1, &v2, &preview_status);
           ci_debug_printf(3, "Preview response was with status: %d \n",
                           preview_status);
-          if (req->eof_received && preview_status == 200) {
+          if (preview_status == 204)
+              ci_headers_unpack(req->response_header);
+          else if ((req->eof_received && preview_status == 200) || preview_status == 206) {
                ci_headers_unpack(req->response_header);
                if ((val = ci_headers_search(req->response_header, "Encapsulated")) == NULL) {
                     ci_debug_printf(1, "No encapsulated entities!\n");
@@ -1218,5 +1364,9 @@ int ci_client_icapfilter(ci_request_t * req,
      ret =
          client_send_get_data(req, timeout, data_source, source_read, data_dest,
                               dest_write);
+
+     if (preview_status == 206 && ret == CI_OK)
+         return 206;
+
      return ret;
 }
