@@ -51,12 +51,13 @@ static int service_aliases_num = 0;
 /**************************************************************/
 /*service globale config table                                */
 
-int cfg_srv_transfer_preview(char *directive, char **argv, void *setdata);
-int cfg_srv_transfer_ignore(char *directive, char **argv, void *setdata);
-int cfg_srv_transfer_complete(char *directive, char **argv, void *setdata);
-int cfg_srv_preview_size(char *directive, char **argv, void *setdata);
-int cfg_srv_max_connections(char *directive, char **argv, void *setdata);
-
+int cfg_srv_transfer_preview(const char *directive, const char **argv, void *setdata);
+int cfg_srv_transfer_ignore(const char *directive, const char **argv, void *setdata);
+int cfg_srv_transfer_complete(const char *directive, const char **argv, void *setdata);
+int cfg_srv_preview_size(const char *directive, const char **argv, void *setdata);
+int cfg_srv_max_connections(const char *directive, const char **argv, void *setdata);
+int cfg_srv_options_ttl(const char *directive, const char **argv, void *setdata);
+int cfg_srv_allow206(const char *directive, const char **argv, void *setdata);
 
 static struct ci_conf_entry services_global_conf_table[] = {
   {"TransferPreview", NULL, cfg_srv_transfer_preview, NULL},
@@ -64,10 +65,12 @@ static struct ci_conf_entry services_global_conf_table[] = {
   {"TransferComplete", NULL, cfg_srv_transfer_complete, NULL},
   {"PreviewSize", NULL, cfg_srv_preview_size, NULL},
   {"MaxConnections", NULL, cfg_srv_max_connections, NULL},
+  {"OptionsTTL", NULL, cfg_srv_options_ttl, NULL},
+  {"Allow206", NULL, cfg_srv_allow206, NULL},
   {NULL, NULL, NULL, NULL}
 };
 
-int cfg_srv_transfer_preview(char *directive, char **argv, void *setdata)
+int cfg_srv_transfer_preview(const char *directive, const char **argv, void *setdata)
 {
     struct ci_service_xdata *srv_xdata = ( struct ci_service_xdata *)setdata;
     if (argv == NULL || argv[0] == NULL) {
@@ -79,7 +82,7 @@ int cfg_srv_transfer_preview(char *directive, char **argv, void *setdata)
     return 1;
 }
 
-int cfg_srv_transfer_ignore(char *directive, char **argv, void *setdata)
+int cfg_srv_transfer_ignore(const char *directive, const char **argv, void *setdata)
 {
     struct ci_service_xdata *srv_xdata = ( struct ci_service_xdata *)setdata;
     if (argv == NULL || argv[0] == NULL) {
@@ -91,7 +94,7 @@ int cfg_srv_transfer_ignore(char *directive, char **argv, void *setdata)
     return 1;
 }
 
-int cfg_srv_transfer_complete(char *directive, char **argv, void *setdata)
+int cfg_srv_transfer_complete(const char *directive, const char **argv, void *setdata)
 {
     struct ci_service_xdata *srv_xdata = ( struct ci_service_xdata *)setdata;
     if (argv == NULL || argv[0] == NULL) {
@@ -103,7 +106,7 @@ int cfg_srv_transfer_complete(char *directive, char **argv, void *setdata)
     return 1;
 }
 
-int cfg_srv_preview_size(char *directive, char **argv, void *setdata)
+int cfg_srv_preview_size(const char *directive, const char **argv, void *setdata)
 {
     int preview;
     char *end;
@@ -112,18 +115,24 @@ int cfg_srv_preview_size(char *directive, char **argv, void *setdata)
 	ci_debug_printf(1, "Missing arguments in directive %s \n", directive);
 	return 0;
     }
-    errno = 0;
-    preview = strtoll(argv[0], &end, 10);
-    if (errno != 0 || preview < 0) {
-	ci_debug_printf(1, "Invalid argument in directive %s \n", directive);
-	return 0;
+    if (strcmp(argv[0], "-") == 0) {
+        preview = -1;
+        ci_debug_printf(2, "Setting parameter :%s=\"disable\"\n", directive);
+    }
+    else {
+        errno = 0;
+        preview = strtoll(argv[0], &end, 10);
+        if (errno != 0 || preview < 0) {
+	    ci_debug_printf(1, "Invalid argument in directive %s \n", directive);
+	    return 0;
+        }
+        ci_debug_printf(2, "Setting parameter :%s=%d\n", directive, preview)
     }
-    ci_debug_printf(2, "Setting parameter :%s=%d\n", directive, preview);
     ci_service_set_preview(srv_xdata, preview);
     return 1;
 }
 
-int cfg_srv_max_connections(char *directive, char **argv, void *setdata)
+int cfg_srv_max_connections(const char *directive, const char **argv, void *setdata)
 {
     int max_connections;
     char *end;
@@ -143,6 +152,71 @@ int cfg_srv_max_connections(char *directive, char **argv, void *setdata)
     return 1;
 }
 
+int cfg_srv_options_ttl(const char *directive, const char **argv, void *setdata)
+{
+    int ttl;
+    int u;
+    char *end = NULL;
+    const char *units;
+    struct ci_service_xdata *srv_xdata = ( struct ci_service_xdata *)setdata;
+    if (argv == NULL || argv[0] == NULL) {
+	ci_debug_printf(1, "Missing arguments in directive %s \n", directive);
+	return 0;
+    }
+    errno = 0;
+    ttl = strtoll(argv[0], &end, 10);
+    if (errno != 0 || ttl < 0) {
+	ci_debug_printf(1, "Invalid argument in directive %s \n", directive);
+	return 0;
+    }
+
+    if (end && *end != '\0')
+        units = end;
+    else if (argv[1] != NULL)
+        units = argv[1];
+    else
+        units = NULL;
+
+    if (units) {
+        if (strcasecmp(units, "sec") == 0 || strcasecmp(units, "secs") == 0)
+            u = 1;
+        else
+        if (strcasecmp(units, "min") == 0 || strcasecmp(units, "mins") == 0)
+            u = 60;
+        else if (strcasecmp(units, "hour") == 0 || strcasecmp(units, "hours") == 0)
+            u = 60*60;
+        else {
+            ci_debug_printf(1, "Invalid units %s,  in directive %s \n", units, directive);
+            return 0;
+        }
+    }
+    else
+        u = 1;
+    
+    ttl = ttl * u;
+    ci_debug_printf(2, "Setting parameter :%s=%d\n", directive, ttl);
+    ci_service_set_options_ttl(srv_xdata, ttl);
+    return 1;
+}
+
+int cfg_srv_allow206(const char *directive, const char **argv, void *setdata)
+{
+    struct ci_service_xdata *srv_xdata = ( struct ci_service_xdata *)setdata;
+    if (argv == NULL || argv[0] == NULL) {
+	ci_debug_printf(1, "Missing arguments in directive %s \n", directive);
+	return 0;
+    }
+    ci_debug_printf(1, "Setting parameter :%s=%s\n", directive, argv[0]);
+    
+    if (strcasecmp(argv[0], "off")) {
+        ci_thread_rwlock_wrlock(&srv_xdata->lock);
+        srv_xdata->disable_206 = 1;
+        ci_thread_rwlock_unlock(&srv_xdata->lock);
+    }
+
+    return 1;
+}
+
 struct ci_conf_entry *create_service_conf_table(struct ci_service_xdata *srv_xdata,struct ci_conf_entry *user_table)
 {
     int i,k,size;
@@ -184,10 +258,10 @@ struct ci_conf_entry *create_service_conf_table(struct ci_service_xdata *srv_xda
  Base functions for services support ....
 *****************************************************************/
 
-ci_service_module_t *find_service_by_alias(char *service_name);
+ci_service_module_t *find_service_by_alias(const char *service_name);
 
 
-ci_service_module_t *create_service(char *service_file)
+ci_service_module_t *create_service(const char *service_file)
 {
      char *extension;
      service_handler_module_t *service_handler;
@@ -200,10 +274,11 @@ ci_service_module_t *create_service(char *service_file)
 
 }
 
-void init_extra_data(ci_service_xdata_t * srv_xdata, char *service)
+void init_extra_data(ci_service_xdata_t * srv_xdata, const char *service)
 {
      char buf[1024];
      char stat_group[1024];
+     memset(srv_xdata, 0, sizeof(ci_service_xdata_t));
      ci_thread_rwlock_init(&srv_xdata->lock);
      strcpy(srv_xdata->ISTag, "ISTag: ");
      strcat(srv_xdata->ISTag, ISTAG "-XXXXXXXXX");
@@ -213,8 +288,12 @@ void init_extra_data(ci_service_xdata_t * srv_xdata, char *service)
      memset(srv_xdata->TransferComplete, 0, MAX_HEADER_SIZE + 1);
      srv_xdata->preview_size = 0;
      srv_xdata->allow_204 = 0;
-     srv_xdata->max_connections = 0;
+     srv_xdata->allow_206 = 0;
+     srv_xdata->disable_206 = 0;
+     srv_xdata->max_connections = -1;
      srv_xdata->xopts = 0;
+     srv_xdata->status = CI_SERVICE_NOT_INITIALIZED;
+     srv_xdata->options_ttl = -1;
 
      snprintf(stat_group, 1023, "Service %s", service);
      stat_group[1023] = '\0';
@@ -261,6 +340,7 @@ ci_service_module_t *add_service(ci_service_module_t *service)
 {
      struct ci_service_xdata *xdata=NULL;
      struct ci_conf_entry *cfg_table;
+     int ret;
 
      if (service_list == NULL) {
           service_list_size = STEP;
@@ -287,8 +367,15 @@ ci_service_module_t *add_service(ci_service_module_t *service)
 
      xdata = &service_extra_data_list[services_num];
      init_extra_data(xdata, service->mod_name);
-     if (service->mod_init_service)
-          service->mod_init_service(xdata, &CONF);
+     if (service->mod_init_service) {
+          ret = service->mod_init_service(xdata, &CONF);
+          if (ret != CI_OK)
+               xdata->status = CI_SERVICE_ERROR;
+          else
+               xdata->status = CI_SERVICE_OK;
+     }
+     else
+          xdata->status = CI_SERVICE_OK;
 
      service_list[services_num++] = service;
 
@@ -302,12 +389,12 @@ ci_service_module_t *add_service(ci_service_module_t *service)
      return service;
 }
 
-ci_service_module_t *register_service(char *service_file)
+ci_service_module_t *register_service(const char *service_file)
 {
      ci_service_module_t *service;
      service = create_service(service_file);
      if (!service) {
-          ci_debug_printf(1, "Error finding symbol \"service\" in  module %s\n",
+          ci_debug_printf(3, "Error while loading service %s\n",
                           service_file);
           return NULL;
      }
@@ -321,7 +408,7 @@ ci_service_module_t *register_service(char *service_file)
 }
 
 
-ci_service_module_t *find_service(char *service_name)
+ci_service_module_t *find_service(const char *service_name)
 {
      int i;
      for (i = 0; i < services_num; i++) {
@@ -355,11 +442,17 @@ int init_services()
 
 int post_init_services()
 {
-     int i;
+     int i, ret;
+     ci_service_xdata_t *xdata;
      for (i = 0; i < services_num; i++) {
           if (service_list[i]->mod_post_init_service != NULL) {
-               service_list[i]->
-                   mod_post_init_service(&service_extra_data_list[i], &CONF);
+               xdata = &service_extra_data_list[i];
+               if ( xdata->status == CI_SERVICE_OK) {
+                    ret = service_list[i]->
+                        mod_post_init_service(xdata, &CONF);
+                    if (ret != CI_OK)
+                        xdata->status = CI_SERVICE_ERROR;
+               }
           }
      }
      return 1;
@@ -388,15 +481,23 @@ int release_services()
 	       service_extra_data_list[i].intl_srv_conf_table = NULL;
           }
      }
+
+     free(service_list);
+     service_list = NULL;
+     service_list_size = 0;
      services_num = 0;
+
+     free(service_aliases);
+     service_aliases = NULL;
+     service_aliases_size = 0;
      service_aliases_num = 0;
      return 1;
 }
 
 
 /********************** Service aliases *****************************/
-service_alias_t *add_service_alias(char *service_alias, char *service_name,
-                                   char *args)
+service_alias_t *add_service_alias(const char *service_alias, const char *service_name,
+                                   const char *args)
 {
      ci_service_module_t *service = NULL;
      service_alias_t *salias = NULL;
@@ -463,7 +564,7 @@ service_alias_t *add_service_alias(char *service_alias, char *service_name,
      return &(service_aliases[alias_indx]);
 }
 
-ci_service_module_t *find_service_by_alias(char *service_name)
+ci_service_module_t *find_service_by_alias(const char *service_name)
 {
      int i;
      for (i = 0; i < service_aliases_num; i++) {
@@ -473,7 +574,7 @@ ci_service_module_t *find_service_by_alias(char *service_name)
      return NULL;
 }
 
-service_alias_t *find_service_alias(char *service_name)
+service_alias_t *find_service_alias(const char *service_name)
 {
      int i;
      for (i = 0; i < service_aliases_num; i++) {
@@ -489,7 +590,7 @@ service_alias_t *find_service_alias(char *service_name)
   and loaded as dynamic libraries
  **********************************************************************/
 
-ci_service_module_t *load_c_service(char *service_file);
+ci_service_module_t *load_c_service(const char *service_file);
 void release_c_handler();
 
 service_handler_module_t c_service_handler = {
@@ -502,7 +603,7 @@ service_handler_module_t c_service_handler = {
      NULL                       /*config table .... */
 };
 
-ci_service_module_t *load_c_service(char *service_file)
+ci_service_module_t *load_c_service(const char *service_file)
 {
      ci_service_module_t *service = NULL;
      CI_DLIB_HANDLE service_handle;
