@@ -19,7 +19,7 @@
 
 #include "common.h"
 #include "simple_api.h"
-
+#include <ctype.h>
 
 
 /*
@@ -54,6 +54,14 @@ ci_headers_list_t *ci_http_request_headers(ci_request_t * req)
      if (e_list[0] != NULL && e_list[0]->type == ICAP_REQ_HDR)  /*It is always the first ellement */
           return (ci_headers_list_t *) e_list[0]->entity;
 
+     /*We did not found the request headers but maybe there are in req->trash objects
+       Trying to retrieve hoping that it were not desstroyed yet
+      */
+     if(req->trash_entities[ICAP_REQ_HDR] &&
+        req->trash_entities[ICAP_REQ_HDR]->entity &&
+        ((ci_headers_list_t *) req->trash_entities[ICAP_REQ_HDR]->entity)->used)
+         return (ci_headers_list_t *) req->trash_entities[ICAP_REQ_HDR]->entity;
+
      return NULL;
 }
 
@@ -128,7 +136,7 @@ int ci_http_request_create(ci_request_t * req, int has_body)
 }
 
 
-char *ci_http_response_add_header(ci_request_t * req, char *header)
+const char *ci_http_response_add_header(ci_request_t * req, const char *header)
 {
      ci_headers_list_t *heads;
      if(req->packed)   /*Not in edit mode*/
@@ -139,7 +147,7 @@ char *ci_http_response_add_header(ci_request_t * req, char *header)
 }
 
 
-char *ci_http_request_add_header(ci_request_t * req, char *header)
+const char *ci_http_request_add_header(ci_request_t * req, const char *header)
 {
      ci_headers_list_t *heads;
      if(req->packed)   /*Not in edit mode*/
@@ -149,7 +157,7 @@ char *ci_http_request_add_header(ci_request_t * req, char *header)
      return ci_headers_add(heads, header);
 }
 
-int ci_http_response_remove_header(ci_request_t * req, char *header)
+int ci_http_response_remove_header(ci_request_t * req, const char *header)
 {
      ci_headers_list_t *heads;
      if(req->packed)   /*Not in edit mode*/
@@ -160,7 +168,7 @@ int ci_http_response_remove_header(ci_request_t * req, char *header)
 }
 
 
-int ci_http_request_remove_header(ci_request_t * req, char *header)
+int ci_http_request_remove_header(ci_request_t * req, const char *header)
 {
      ci_headers_list_t *heads;
      if(req->packed)   /*Not in edit mode*/
@@ -171,10 +179,10 @@ int ci_http_request_remove_header(ci_request_t * req, char *header)
 }
 
 
-char *ci_http_response_get_header(ci_request_t * req, char *head_name)
+const char *ci_http_response_get_header(ci_request_t * req, const char *head_name)
 {
      ci_headers_list_t *heads;
-     char *val;
+     const char *val;
      if (!(heads =  ci_http_response_headers(req)))
           return NULL;
      if (!(val = ci_headers_value(heads, head_name)))
@@ -182,10 +190,10 @@ char *ci_http_response_get_header(ci_request_t * req, char *head_name)
      return val;
 }
 
-char *ci_http_request_get_header(ci_request_t * req, char *head_name)
+const char *ci_http_request_get_header(ci_request_t * req, const char *head_name)
 {
      ci_headers_list_t *heads;
-     char *val;
+     const char *val;
      if (!(heads = ci_http_request_headers(req)))
           return NULL;
      if (!(val = ci_headers_value(heads, head_name)))
@@ -197,7 +205,7 @@ char *ci_http_request_get_header(ci_request_t * req, char *head_name)
 ci_off_t ci_http_content_length(ci_request_t * req)
 {
      ci_headers_list_t *heads;
-     char *val;
+     const char *val;
      if (!(heads =  ci_http_response_headers(req))) {
           /*Then maybe is a reqmod reauest, try to get request headers */
           if (!(heads = ci_http_request_headers(req)))
@@ -208,7 +216,7 @@ ci_off_t ci_http_content_length(ci_request_t * req)
      return ci_strto_off_t(val, NULL, 10);
 }
 
-char *ci_http_request(ci_request_t * req)
+const char *ci_http_request(ci_request_t * req)
 {
      ci_headers_list_t *heads;
      if (!(heads = ci_http_request_headers(req)))
@@ -220,7 +228,7 @@ char *ci_http_request(ci_request_t * req)
      return heads->headers[0];
 }
 
-char *ci_icap_add_xheader(ci_request_t * req, char *header)
+const char *ci_icap_add_xheader(ci_request_t * req, const char *header)
 {
      return ci_headers_add(req->xheaders, header);
 }
@@ -230,11 +238,12 @@ int ci_icap_append_xheaders(ci_request_t *req,ci_headers_list_t *headers)
     return ci_headers_addheaders(req->xheaders, headers);
 }
 
+#define header_end(e) (e == '\0' || e == '\n' || e == '\r')
 int ci_http_request_url(ci_request_t * req, char *buf, int buf_size)
 {
    ci_headers_list_t *heads;
-   char *str;
-   int i; 
+   const char *str, *host;
+   int i, bytes; 
    /*The request must have the form:
         GET url HTTP/X.X 
    */
@@ -252,11 +261,24 @@ int ci_http_request_url(ci_request_t * req, char *buf, int buf_size)
      while (*str == ' ') /*ignore spaces*/
           str++;
 
+     bytes = 0;
+     if (*str == '/' && (host = ci_headers_value(heads,"Host"))) {
+         /*Looks like a transparent proxy, we do not know the protocol lets try
+           to preserve the major part of the url....
+          */
+         for (i=0; (i < buf_size-1) && !header_end(host[i]) && !isspace(host[i]); i++) {
+             buf[i] = host[i]; 
+         }
+         buf += i;
+         buf_size -= i;
+         bytes = i;
+     }
+
      /*copy the url...*/
-     for (i=0; i < buf_size-1 && (str[i] !=' ' && str[i] != '\0' && str[i] != '\n' && str[i] != '\r' && str[i] != '?'); 
-             i++) {
+     for (i=0; (i < buf_size-1) && !header_end(str[i]) && !isspace(str[i]) && str[i] != '?'; i++) {
           buf[i] = str[i]; 
      }
      buf[i] = '\0';
-     return i;
+     bytes += i;
+     return bytes;
 }
