@@ -27,7 +27,9 @@
 #include <sys/param.h>
 #include <assert.h>
 #include <stdlib.h>
+#include <ctype.h>
 
+#include "common.h" 
 #include "body.h"
 #include "c-icap.h"
 #include "service.h"
@@ -289,8 +291,10 @@ static txtTemplate_t *templateTryLoadText(const ci_request_t * req, const char *
      /* TODO: do not allow txttemplates bigger than 64k 
       */
      textbuff = ci_membuf_new_sized(file.st_size + 1);
-     
-     assert(textbuff != NULL);
+     if (!textbuff) {
+         ci_debug_printf(1, "templateTryLoadText: membuf allocation failed!\n");
+         return NULL;
+     }
      
      while ((len = read(fd, buf, sizeof(buf))) > 0) {
 	  ci_membuf_write(textbuff, buf, len, 0);
@@ -319,6 +323,7 @@ static txtTemplate_t *templateTryLoadText(const ci_request_t * req, const char *
 	  if (!tempTemplate) {
 	       ci_debug_printf(1, "templateTryLoadText: memory allocation error!\n");
 	       ci_thread_mutex_unlock(&templates_mutex);
+               ci_membuf_free(textbuff);
 	       return NULL;
 	  }
 	  tempTemplate->non_cached = 1;
@@ -391,23 +396,32 @@ static txtTemplate_t *templateLoadText(const ci_request_t * req, const char *ser
 ci_membuf_t *ci_txt_template_build_content(const ci_request_t *req, const char *SERVICE_NAME,
                                const char *TEMPLATE_NAME, struct ci_fmt_entry *user_table)
 {
-     ci_membuf_t *content = ci_membuf_new(TEMPLATE_MEMBUF_SIZE);
+    ci_membuf_t *content;
      char templpath[CI_MAX_PATH];
      txtTemplate_t *template = NULL;
 
+     content = ci_membuf_new(TEMPLATE_MEMBUF_SIZE);
+     if (!content) {
+         ci_debug_printf(1, "Failed to allocate buffer to load template!");
+         return NULL;
+     }
+
      /*templateLoadText also locks the template*/
      template = templateLoadText(req, SERVICE_NAME, TEMPLATE_NAME);
      if (template) {
-       content->endpos = ci_format_text((ci_request_t *)req, template->data->buf, content->buf, content->bufsize, user_table);
-          template_release(template);
+         content->endpos = ci_format_text((ci_request_t *)req, template->data->buf, content->buf, content->bufsize, user_table);
+         ci_membuf_write(content, "\0", 1, 1);      // terminate the string for safety (????)
+         if (template->LANGUAGE)
+             ci_membuf_attr_add(content, "lang", template->LANGUAGE, strlen(template->LANGUAGE) + 1);
 
-          ci_membuf_write(content, "\0", 1, 1);      // terminate the string for safety
+          template_release(template);
      }
      else {
           makeTemplatePathFileName(templpath, CI_MAX_PATH, SERVICE_NAME, TEMPLATE_NAME, TEMPLATE_DEF_LANG);
           content->endpos = snprintf(content->buf, content->bufsize, "ERROR: Unable to find specified template: %s\n", templpath);
           if (content->endpos > content->bufsize)
               content->endpos = content->bufsize;
+          ci_membuf_attr_add(content, "lang", TEMPLATE_DEF_LANG, strlen(TEMPLATE_DEF_LANG) + 1);
           ci_debug_printf(1, "ERROR: Unable to find specified template: %s\n", templpath);
      }
 
