@@ -26,32 +26,35 @@
 
 #define MAX_VARIABLE_SIZE 256
 
-int fmt_none(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_percent(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_remoteip(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_localip(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_icapstatus(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_icapmethod(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_service(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_username(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_request(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_localtime(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_gmttime(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_seconds(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_httpclientip(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_httpserverip(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_http_req_url_o(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_http_req_head_o(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_http_res_head_o(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_icap_req_head(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_icap_res_head(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_req_bytes_rcv(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_req_bytes_sent(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_req_http_bytes_rcv(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_req_http_bytes_sent(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_req_body_bytes_rcv(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_req_body_bytes_sent(ci_request_t *req_data, char *buf,int len, char *param);
-int fmt_req_preview_hex(ci_request_t *req_data, char *buf,int len, char *param);
+int fmt_none(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_percent(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_remoteip(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_localip(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_icapstatus(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_icapmethod(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_service(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_username(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_request(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_localtime(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_gmttime(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_seconds(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_httpclientip(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_httpserverip(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_http_req_url_o(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_http_req_head_o(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_http_res_head_o(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_icap_req_head(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_icap_res_head(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_req_bytes_rcv(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_req_bytes_sent(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_req_http_bytes_rcv(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_req_http_bytes_sent(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_req_body_bytes_rcv(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_req_body_bytes_sent(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_req_preview_hex(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_req_preview_len(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_logstr(ci_request_t *req_data, char *buf,int len, const char *param);
+int fmt_req_attribute(ci_request_t *req_data, char *buf,int len, const char *param);
 
 /**
    \brief Internal formating directives table.
@@ -74,6 +77,7 @@ int fmt_req_preview_hex(ci_request_t *req_data, char *buf,int len, char *param);
    * \em "%is": Icap status code \n
    * \em "%>ih": Icap request header \n
    * \em "%<ih": Icap response header \n
+   * \em "%ipl": Icap preview length \n
    * \em "%Ih": Http bytes received \n
    * \em "%Oh": Http bytes sent \n
    * \em "%Ib": Http body bytes received \n
@@ -82,6 +86,8 @@ int fmt_req_preview_hex(ci_request_t *req_data, char *buf,int len, char *param);
    * \em "%O": Bytes sent \n
    * \em "%bph": Body data preview \n
    * \em "%un": Username \n
+   * \em "%Sl": Log string set by service\n
+   * \em "%Sa": Attribute value set by service\n
    *
    * Not yet implemented:\n
    * \em "%tr": Response time \n
@@ -115,6 +121,7 @@ struct ci_fmt_entry GlobalTable [] = {
     {"%is", "Icap status code", fmt_icapstatus},
     {"%>ih", "Icap request header", fmt_icap_req_head},
     {"%<ih", "Icap response header", fmt_icap_res_head},
+    {"%ipl", "Icap preview length", fmt_req_preview_len},
     
     {"%Ih", "Http bytes received", fmt_req_http_bytes_rcv},
     {"%Oh", "Http bytes sent", fmt_req_http_bytes_sent},
@@ -126,11 +133,13 @@ struct ci_fmt_entry GlobalTable [] = {
 
     {"%bph", "Body data preview", fmt_req_preview_hex},
     {"%un", "Username", fmt_username},
+    {"%Sl", "Service log string", fmt_logstr},
+    {"%Sa", "Attribute set by service", fmt_req_attribute},
     {"%%", "% sign", fmt_percent},
     { NULL, NULL, NULL} 
 };
 
-int fmt_none(ci_request_t *req, char *buf,int len, char *param)
+int fmt_none(ci_request_t *req, char *buf,int len, const char *param)
 {
   if (!len)
      return 0;
@@ -139,7 +148,7 @@ int fmt_none(ci_request_t *req, char *buf,int len, char *param)
    return 1;
 }
 
-int fmt_percent(ci_request_t *req, char *buf,int len, char *param)
+int fmt_percent(ci_request_t *req, char *buf,int len, const char *param)
 {
    if (!len)
       return 0;
@@ -241,6 +250,7 @@ int ci_format_text(
    unsigned int width, space=0;
    char parameter[MAX_VARIABLE_SIZE];
 
+   lb = NULL;
    s = fmt;
    b = buffer;
    remains = len;
@@ -257,14 +267,17 @@ int ci_format_text(
             if (width) {
                 if (left_align) {
                     val_len=fmte->format(req_data, b, space, parameter);
+                    if (val_len <=0)  val_len = fmt_none(req_data, b, space, parameter);
+                    
                     if (val_len > space) val_len = space;
                     b += val_len;
                     for (i=0; i < width-val_len; i++) b[i]=' ';
                     b += width-val_len;
                 }
-                else {
-                    lb = malloc((space+1)*sizeof(char));      
+                else if ((lb = malloc((space+1)*sizeof(char))) != NULL) {
                     val_len=fmte->format(req_data, lb, space, parameter);
+                    if (val_len <=0)  val_len = fmt_none(req_data, lb, space, parameter);
+
                     if (val_len > space) val_len = space;
                     for (i=0; i < width-val_len; i++) b[i]=' ';
                     b += width-val_len; 
@@ -273,11 +286,14 @@ int ci_format_text(
                     free(lb);
                     lb = NULL;
                 }
+                /*else allocation failed! Just ignore?????*/
 
                 remains -= width;
             } 
              else {
 		val_len=fmte->format(req_data, b, space, parameter);
+                if (val_len <=0)  val_len = fmt_none(req_data, b, space, parameter);
+
                 if (val_len > space) val_len = space;
                 b += val_len; 
                 remains -= val_len;
@@ -297,7 +313,7 @@ int ci_format_text(
 
 /******************************************************************/
 
-int fmt_remoteip(ci_request_t *req, char *buf,int len, char *param)
+int fmt_remoteip(ci_request_t *req, char *buf,int len, const char *param)
 {
     if (len<CI_IPLEN)
 	return 0;
@@ -308,7 +324,7 @@ int fmt_remoteip(ci_request_t *req, char *buf,int len, char *param)
     return strlen(buf);
 }
 
-int fmt_localip(ci_request_t *req, char *buf,int len, char *param)
+int fmt_localip(ci_request_t *req, char *buf,int len, const char *param)
 {
     if (len<CI_IPLEN)
         return 0;
@@ -319,7 +335,7 @@ int fmt_localip(ci_request_t *req, char *buf,int len, char *param)
     return strlen(buf);
 }
 
-int fmt_icapmethod(ci_request_t *req, char *buf,int len, char *param)
+int fmt_icapmethod(ci_request_t *req, char *buf,int len, const char *param)
 {
    int i;
    const char *s = ci_method_string(req->type);
@@ -328,7 +344,7 @@ int fmt_icapmethod(ci_request_t *req, char *buf,int len, char *param)
    return i;
 }
 
-int fmt_service(ci_request_t *req, char *buf,int len, char *param)
+int fmt_service(ci_request_t *req, char *buf,int len, const char *param)
 {
    int i;
    char *s = req->service;
@@ -337,7 +353,7 @@ int fmt_service(ci_request_t *req, char *buf,int len, char *param)
    return i;
 }
 
-int fmt_username(ci_request_t *req, char *buf,int len, char *param)
+int fmt_username(ci_request_t *req, char *buf,int len, const char *param)
 {
    int i;
    char *s = req->user;
@@ -346,7 +362,7 @@ int fmt_username(ci_request_t *req, char *buf,int len, char *param)
    return i;
 }
 
-int fmt_request(ci_request_t *req, char *buf,int len, char *param)
+int fmt_request(ci_request_t *req, char *buf,int len, const char *param)
 {
    int i;
    char *s = req->service;
@@ -363,11 +379,11 @@ int fmt_request(ci_request_t *req, char *buf,int len, char *param)
    return i;
 }
 
-int fmt_localtime(ci_request_t *req, char *buf,int len, char *param)
+int fmt_localtime(ci_request_t *req, char *buf,int len, const char *param)
 {
     struct tm tm;
     time_t t;
-    char *tfmt = "%d/%b/%Y:%H:%M:%S %z";
+    const char *tfmt = "%d/%b/%Y:%H:%M:%S %z";
 
     if (!len)
         return 0;
@@ -380,11 +396,11 @@ int fmt_localtime(ci_request_t *req, char *buf,int len, char *param)
     return strftime(buf, len, tfmt, &tm);
 }
 
-int fmt_gmttime(ci_request_t *req, char *buf,int len, char *param)
+int fmt_gmttime(ci_request_t *req, char *buf,int len, const char *param)
 {
     struct tm tm;
     time_t t;
-    char *tfmt = "%d/%b/%Y:%H:%M:%S";
+    const char *tfmt = "%d/%b/%Y:%H:%M:%S";
 
     if (!len)
         return 0;
@@ -397,22 +413,22 @@ int fmt_gmttime(ci_request_t *req, char *buf,int len, char *param)
     return strftime(buf, len, tfmt, &tm);
 }
 
-int fmt_icapstatus(ci_request_t *req, char *buf,int len, char *param)
+int fmt_icapstatus(ci_request_t *req, char *buf,int len, const char *param)
 {
    return snprintf(buf, len, "%d", ci_error_code(req->return_code));
 }
 
 
-int fmt_seconds(ci_request_t *req, char *buf,int len, char *param)
+int fmt_seconds(ci_request_t *req, char *buf,int len, const char *param)
 {
    time_t tm;
    time(&tm);
    return snprintf(buf, len, "%ld", tm);
 }
 
-int fmt_httpclientip(ci_request_t *req, char *buf,int len, char *param)
+int fmt_httpclientip(ci_request_t *req, char *buf,int len, const char *param)
 {
-    char *s;
+  const char *s;
   int i;
   if (!len)
      return 0;
@@ -429,9 +445,9 @@ int fmt_httpclientip(ci_request_t *req, char *buf,int len, char *param)
 
 }
 
-int fmt_httpserverip(ci_request_t *req, char *buf,int len, char *param)
+int fmt_httpserverip(ci_request_t *req, char *buf,int len, const char *param)
 {
-  char *s;
+  const char *s;
   int i;
   if (!len)
      return 0;
@@ -447,7 +463,7 @@ int fmt_httpserverip(ci_request_t *req, char *buf,int len, char *param)
   }
 }
 
-int fmt_http_req_url_o(ci_request_t *req, char *buf,int len, char *param)
+int fmt_http_req_url_o(ci_request_t *req, char *buf,int len, const char *param)
 {
     if (!len)
         return 0;
@@ -455,9 +471,9 @@ int fmt_http_req_url_o(ci_request_t *req, char *buf,int len, char *param)
      return ci_http_request_url(req, buf, len);
 }
 
-int fmt_http_req_head_o(ci_request_t *req, char *buf,int len, char *param)
+int fmt_http_req_head_o(ci_request_t *req, char *buf,int len, const char *param)
 {
-  char *s = NULL;
+  const char *s = NULL;
   int i;
   if (!len)
      return 0;
@@ -478,9 +494,9 @@ int fmt_http_req_head_o(ci_request_t *req, char *buf,int len, char *param)
   }
 }
 
-int fmt_http_res_head_o(ci_request_t *req, char *buf,int len, char *param)
+int fmt_http_res_head_o(ci_request_t *req, char *buf,int len, const char *param)
 {
-  char *s = NULL;
+  const char *s = NULL;
   int i;
   ci_headers_list_t *http_resp_headers;
 
@@ -506,9 +522,9 @@ int fmt_http_res_head_o(ci_request_t *req, char *buf,int len, char *param)
 }
 
 
-int fmt_icap_req_head(ci_request_t *req, char *buf,int len, char *param)
+int fmt_icap_req_head(ci_request_t *req, char *buf,int len, const char *param)
 {
-  char *s = NULL;
+  const char *s = NULL;
   int i;
   if (!len)
      return 0;
@@ -530,9 +546,9 @@ int fmt_icap_req_head(ci_request_t *req, char *buf,int len, char *param)
   }
 }
 
-int fmt_icap_res_head(ci_request_t *req, char *buf,int len, char *param)
+int fmt_icap_res_head(ci_request_t *req, char *buf,int len, const char *param)
 {
-  char *s = NULL;
+  const char *s = NULL;
   int i;
   if (!len)
      return 0;
@@ -542,6 +558,9 @@ int fmt_icap_res_head(ci_request_t *req, char *buf,int len, char *param)
           s = req->response_header->headers[0];
   } else {
       s = ci_headers_value(req->response_header, param);
+      /*if not found try xheaders which will also sent to user*/
+      if (!s && req->xheaders)
+          s = ci_headers_value(req->xheaders, param);
   }
 
   if (s) {
@@ -555,31 +574,31 @@ int fmt_icap_res_head(ci_request_t *req, char *buf,int len, char *param)
 }
 
 
-int fmt_req_bytes_rcv(ci_request_t *req, char *buf,int len, char *param) {
+int fmt_req_bytes_rcv(ci_request_t *req, char *buf,int len, const char *param) {
     return snprintf(buf, len, "%" PRINTF_OFF_T , (CAST_OFF_T) req->bytes_in);
 }
 
-int fmt_req_bytes_sent(ci_request_t *req, char *buf,int len, char *param) {
+int fmt_req_bytes_sent(ci_request_t *req, char *buf,int len, const char *param) {
     return snprintf(buf, len, "%" PRINTF_OFF_T , (CAST_OFF_T) req->bytes_out);
 }
 
-int fmt_req_http_bytes_rcv(ci_request_t *req, char *buf,int len, char *param) {
+int fmt_req_http_bytes_rcv(ci_request_t *req, char *buf,int len, const char *param) {
     return snprintf(buf, len, "%" PRINTF_OFF_T , (CAST_OFF_T) req->http_bytes_in);
 }
 
-int fmt_req_http_bytes_sent(ci_request_t *req, char *buf,int len, char *param) {
+int fmt_req_http_bytes_sent(ci_request_t *req, char *buf,int len, const char *param) {
     return snprintf(buf, len, "%" PRINTF_OFF_T , (CAST_OFF_T) req->http_bytes_out);
 }
 
-int fmt_req_body_bytes_rcv(ci_request_t *req, char *buf,int len, char *param) {
+int fmt_req_body_bytes_rcv(ci_request_t *req, char *buf,int len, const char *param) {
     return snprintf(buf, len, "%" PRINTF_OFF_T , (CAST_OFF_T) req->body_bytes_in);
 }
 
-int fmt_req_body_bytes_sent(ci_request_t *req, char *buf,int len, char *param) {
+int fmt_req_body_bytes_sent(ci_request_t *req, char *buf,int len, const char *param) {
     return snprintf(buf, len, "%" PRINTF_OFF_T , (CAST_OFF_T) req->body_bytes_out);
 }
 
-int fmt_req_preview_hex(ci_request_t *req, char *buf,int len, char *param)
+int fmt_req_preview_hex(ci_request_t *req, char *buf,int len, const char *param)
 {
     int  i, num, n, bytes; 
 
@@ -610,3 +629,43 @@ int fmt_req_preview_hex(ci_request_t *req, char *buf,int len, char *param)
     }
     return n;
 }
+
+int fmt_req_preview_len(ci_request_t *req, char *buf, int len, const char *param)
+{
+    if (!len)
+        return 0;
+
+    if (req->preview >=0 )
+        return snprintf(buf, len, "%d", req->preview_data.used);
+    
+    *buf = '-';
+    return 1;
+}
+
+int fmt_logstr(ci_request_t *req, char *buf,int len, const char *param)
+{
+   int i;
+   const char *s;
+
+   if (!req->log_str)
+       return 0;
+
+   s = req->log_str;
+   for(i=0;i<len && *s;i++,s++)
+        buf[i] = *s;
+   return i;
+}
+
+int fmt_req_attribute(ci_request_t *req, char *buf,int len, const char *param)
+{
+    int i;
+    const char *s;
+    if (!req->attributes)
+        return 0;
+    if (! (s =ci_str_array_search(req->attributes, param)))
+        return 0;
+
+    for(i=0; i<len && *s; i++, s++)
+        buf[i] = *s;
+    return i;
+}
