@@ -38,21 +38,21 @@ void *stringdup(const char *str, ci_mem_allocator_t *allocator)
     return new_s;
 }
 
-int stringcmp(void *key1,void *key2)
+int stringcmp(const void *key1,const void *key2)
 {
     if (!key2)
         return -1;
-    return strcmp((char *)key1,(char *)key2);
+    return strcmp((const char *)key1,(const char *)key2);
 }
 
-int stringequal(void *key1,void *key2)
+int stringequal(const void *key1,const void *key2)
 {
     if (!key2)
         return 0;
-    return strcmp((char *)key1,(char *)key2)==0;
+    return strcmp((const char *)key1,(const char *)key2)==0;
 }
 
-size_t stringlen(void *key)
+size_t stringlen(const void *key)
 {
     return strlen((const char *)key)+1;
 }
@@ -70,26 +70,26 @@ ci_type_ops_t  ci_str_ops = {
     stringequal,
 };
 
-int string_ext_cmp(void *key1,void *key2)
+int string_ext_cmp(const void *key1,const void *key2)
 {
     if (!key2)
         return -1;
 
-    if (strcmp(key1, "*") == 0)
+    if (strcmp((const char *)key1, "*") == 0)
 	return 0;
 
-    return strcmp((char *)key1,(char *)key2);
+    return strcmp((const char *)key1,(const char *)key2);
 }
 
-int string_ext_equal(void *key1,void *key2)
+int string_ext_equal(const void *key1,const void *key2)
 {
     if (!key2)
         return 0;
 
-    if (strcmp(key1, "*") == 0)
+    if (strcmp((const char *)key1, "*") == 0)
 	return 1;
 
-    return strcmp((char *)key1,(char *)key2)==0;
+    return strcmp((const char *)key1,(const char *)key2)==0;
 }
 
 
@@ -106,22 +106,36 @@ ci_type_ops_t ci_str_ext_ops = {
 
 void *int32_dup(const char *str, ci_mem_allocator_t *allocator)
 {
-    long i;
-    i = strtol(str, NULL, 10);
+    int32_t *i;
+    i = allocator->alloc(allocator, sizeof(int32_t));
+    if (i)
+        *i = strtol(str, NULL, 10);    
     return (void *)i;
+
 }
 
-int int32_cmp(void *key1,void *key2)
+int int32_cmp(const void *key1,const void *key2)
 {
-    return (key1-key2);
+    int32_t k1, k2;
+    k1 = *(int32_t *)key1;
+    k2 = *(int32_t *)key2;
+    if (k1 < k2)
+        return -1;
+    if (k1 > k2)
+        return 1;
+
+    return 0;
 }
 
-int int32_equal(void *key1,void *key2)
+int int32_equal(const void *key1,const void *key2)
 {
-    return key1 == key2;
+    int32_t k1,k2;
+    k1 = *(int32_t *)key1;
+    k2 = *(int32_t *)key2;
+    return k1 == k2;
 }
 
-size_t int32_len(void *key)
+size_t int32_len(const void *key)
 {
     return (size_t)4;
 }
@@ -129,6 +143,7 @@ size_t int32_len(void *key)
 void int32_free(void *key, ci_mem_allocator_t *allocator)
 {
     /*nothing*/
+    allocator->free(allocator, key);
 }
 
 ci_type_ops_t  ci_int32_ops = {
@@ -139,6 +154,56 @@ ci_type_ops_t  ci_int32_ops = {
     int32_equal
 };
 
+/*uint64 operators*/
+void *uint64_dup(const char *str, ci_mem_allocator_t *allocator)
+{
+    uint64_t *i;
+    i = allocator->alloc(allocator, sizeof(uint64_t));
+    if (i)
+        *i = strtoll(str, NULL, 10);    
+    return (void *)i;
+}
+
+int uint64_cmp(const void *key1,const void *key2)
+{
+    uint64_t k1,k2;
+    k1 = *(uint64_t *)key1;
+    k2 = *(uint64_t *)key2;
+    if (k1 < k2)
+        return -1;
+    if (k1 > k2)
+        return 1;
+
+    return 0;
+}
+
+int uint64_equal(const void *key1,const void *key2)
+{
+   uint64_t k1,k2;
+    k1 = *(uint64_t *)key1;
+    k2 = *(uint64_t *)key2;
+    return k1 == k2;
+}
+
+void uint64_free(void *key, ci_mem_allocator_t *allocator)
+{
+    allocator->free(allocator, key);
+}
+
+size_t uint64_len(const void *key)
+{
+    return (size_t)sizeof(uint64_t);
+}
+
+ci_type_ops_t  ci_uint64_ops = {
+    uint64_dup,
+    uint64_free,
+    uint64_cmp,
+    uint64_len,
+    uint64_equal
+};
+
+
 /*regular expresion operator definition  */
 #ifdef USE_REGEX
 /*We only need the preg field which holds the compiled regular expression 
@@ -218,7 +283,7 @@ void *regex_dup(const char *str, ci_mem_allocator_t *allocator)
     return reg;
 }
 
-int regex_cmp(void *key1,void *key2)
+int regex_cmp(const void *key1, const void *key2)
 {
     regmatch_t pmatch[1];
     struct ci_regex *reg=(struct ci_regex *)key1;
@@ -227,16 +292,16 @@ int regex_cmp(void *key1,void *key2)
     return regexec(&reg->preg, (char *)key2, 1, pmatch, 0);
 }
 
-int regex_equal(void *key1,void *key2)
+int regex_equal(const void *key1, const void *key2)
 {
     regmatch_t pmatch[1];
     struct ci_regex *reg=(struct ci_regex *)key1;
     if (!key2)
         return 0;
-    return regexec(&reg->preg, (char *)key2, 1, pmatch, 0)==0;
+    return regexec(&reg->preg, (const char *)key2, 1, pmatch, 0)==0;
 }
 
-size_t regex_len(void *key)
+size_t regex_len(const void *key)
 {
     return strlen(((const struct ci_regex *)key)->str);
 }
@@ -279,7 +344,7 @@ void *datatype_dup(const char *str, ci_mem_allocator_t *allocator)
     return (void *)val;
 }
 
-int datatype_cmp(void *key1, void *key2)
+int datatype_cmp(const void *key1, const void *key2)
 {
     unsigned int type = *(unsigned int *)key1;
 
@@ -297,7 +362,7 @@ int datatype_cmp(void *key1, void *key2)
     }
 }
 
-int datatype_equal(void *key1, void *key2)
+int datatype_equal(const void *key1, const void *key2)
 {
     unsigned int type = *(unsigned int *)key1;
 
@@ -315,7 +380,7 @@ int datatype_equal(void *key1, void *key2)
     }
 }
 
-size_t datatype_len(void *key)
+size_t datatype_len(const void *key)
 {
     return sizeof(unsigned int);
 }
@@ -464,19 +529,19 @@ void ip_free(void *data, ci_mem_allocator_t *allocator) {
     allocator->free(allocator, data);
 }
 
-size_t ip_len(void *key)
+size_t ip_len(const void *key)
 {
     return sizeof(ci_ip_t);
 }
 
-int ip_cmp(void *ref_key, void *key_check) {
+int ip_cmp(const void *ref_key, const void *key_check) {
     /*Not implemented*/
     return 0;
 }
 
-int ip_equal(void *ref_key, void *key_check) {
-    const ci_ip_t *ip_ref = (ci_ip_t *)ref_key;
-    ci_ip_t *ip_check = (ci_ip_t *)key_check;
+int ip_equal(const void *ref_key, const void *key_check) {
+    const ci_ip_t *ip_ref = (const ci_ip_t *)ref_key;
+    const ci_ip_t *ip_check = (const ci_ip_t *)key_check;
     char buf[128],buf1[128],buf2[128];
 
     if (!ip_check)
@@ -505,14 +570,14 @@ int ip_equal(void *ref_key, void *key_check) {
 
 }
 
-int ip_sockaddr_cmp(void *ref_key, void *key_check) {
+int ip_sockaddr_cmp(const void *ref_key, const void *key_check) {
     /*Not implemented*/
     return 1;
 }
 
-int ip_sockaddr_equal(void *ref_key, void *key_check) {
-    const ci_ip_t *ip_ref = (ci_ip_t *)ref_key;
-    ci_sockaddr_t *ip_check = (ci_sockaddr_t *)key_check;
+int ip_sockaddr_equal(const void *ref_key, const void *key_check) {
+    const ci_ip_t *ip_ref = (const ci_ip_t *)ref_key;
+    const ci_sockaddr_t *ip_check = (const ci_sockaddr_t *)key_check;
     char buf[128],buf1[128],buf2[128];
 
     if (!ip_check)
