@@ -61,12 +61,8 @@ TCHAR *C_ICAP_CMD = TEXT("c-icap.exe -c");
 
 extern int KEEPALIVE_TIMEOUT;
 extern int MAX_SECS_TO_LINGER;
-extern int START_CHILDS;
-extern int MAX_CHILDS;
-extern int START_SERVERS;
-extern int MIN_FREE_SERVERS;
-extern int MAX_FREE_SERVERS;
 extern int MAX_REQUESTS_BEFORE_REALLOCATE_MEM;
+extern struct ci_server_conf CONF;
 ci_socket LISTEN_SOCKET;
 
 #define hard_close_connection(connection)  ci_hard_close(connection->fd)
@@ -344,11 +340,11 @@ void child_main(ci_socket sockfd)
 
 
      threads_list =
-         (server_decl_t **) malloc((START_SERVERS + 1) *
+         (server_decl_t **) malloc((CONF.THREADS_PER_CHILD + 1) *
                                    sizeof(server_decl_t *));
-     con_queue = init_queue(START_SERVERS);
+     con_queue = init_queue(CONF.THREADS_PER_CHILD);
 
-     for (i = 0; i < START_SERVERS; i++) {
+     for (i = 0; i < CONF.THREADS_PER_CHILD; i++) {
           if ((threads_list[i] = newthread(con_queue)) == NULL) {
                exit(-1);        // FATAL error.....
           }
@@ -356,7 +352,7 @@ void child_main(ci_socket sockfd)
                                      (void *(*)(void *)) thread_main,
                                      (void *) threads_list[i]);
      }
-     threads_list[START_SERVERS] = NULL;
+     threads_list[CONF.THREADS_PER_CHILD] = NULL;
      ci_debug_printf(1, "Threads created ....\n");
      retcode = ci_thread_create(&worker_thread,
                                 (void *(*)(void *)) worker_main,
@@ -618,7 +614,7 @@ int do_child()
      }
      ci_debug_printf(1, "Shared memory attached....\n");
      child_data =
-         register_child(&childs_queue, child_handle, START_SERVERS,
+         register_child(&childs_queue, child_handle, CONF.THREADS_PER_CHILD,
                         parent_pipe);
      ci_debug_printf(1, "child registered ....\n");
 
@@ -706,7 +702,7 @@ int check_for_died_child(DWORD msecs)
                break;
      }
      if (count == 0) {
-          ci_debug_printf(1, "Oups no childs! waiting for a while.....\n!");
+          ci_debug_printf(1, "Oups no children! waiting for a while.....\n!");
           Sleep(1000);
           return 0;
      }
@@ -755,14 +751,14 @@ int start_server()
      ci_proc_mutex_init(&accept_mutex);
      ci_thread_mutex_init(&control_process_mtx);
 
-     if (!create_childs_queue(&childs_queue, MAX_CHILDS)) {
+     if (!create_childs_queue(&childs_queue, CONF.MAX_SERVERS)) {
           log_server(NULL, "Can't init shared memory.Fatal error, exiting!\n");
           ci_debug_printf(1,
                           "Can't init shared memory.Fatal error, exiting!\n");
           exit(0);
      }
 
-     for (i = 0; i < START_CHILDS + 2; i++) {
+     for (i = 0; i < CONF.START_SERVERS + 2; i++) {
           child_handle = start_child(LISTEN_SOCKET);
      }
 
@@ -778,15 +774,15 @@ int start_server()
           childs_queue_stats(&childs_queue, &childs, &freeservers, &used,
                              &maxrequests);
           ci_debug_printf(1,
-                          "Server stats: \n\t Childs:%d\n\t Free servers:%d\n\tUsed servers:%d\n\tRequests served:%d\n",
+                          "Server stats: \n\t Children:%d\n\t Free servers:%d\n\tUsed servers:%d\n\tRequests served:%d\n",
                           childs, freeservers, used, maxrequests);
 
-          if ((freeservers <= MIN_FREE_SERVERS && childs < MAX_CHILDS)
-              || childs < START_CHILDS) {
+          if ((freeservers <= CONF.MIN_SPARE_THREADS && childs < CONF.MAX_SERVERS)
+              || childs < CONF.START_SERVERS) {
                ci_debug_printf(1, "Going to start a child .....\n");
                child_handle = start_child(LISTEN_SOCKET);
           }
-          else if (freeservers >= MAX_FREE_SERVERS && childs > START_CHILDS) {
+          else if (freeservers >= CONF.MAX_SPARE_THREADS && childs > CONF.START_SERVERS) {
                ci_thread_mutex_lock(&control_process_mtx);
                if ((child_indx = find_an_idle_child(&childs_queue)) < 0)
                     continue;
@@ -798,7 +794,7 @@ int start_server()
           }
      }
 /*
-     for(i=0;i<START_CHILDS;i++){
+     for(i=0;i<CONF.START_SERVERS;i++){
 	  pid=wait(&status);
 	  ci_debug_printf(1,"The child %d died with status %d\n",pid,status);
      }
@@ -808,7 +804,7 @@ int start_server()
 #else
      child_data = (child_shared_data_t *) malloc(sizeof(child_shared_data_t));
      child_data->pid = 0;
-     child_data->freeservers = START_SERVERS;
+     child_data->freeservers = CONF.THREADS_PER_CHILD;
      child_data->usedservers = 0;
      child_data->requests = 0;
      child_data->connections = 0;
